/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import java.util.Enumeration;
import java.util.Vector;

import static abmland.common.Utils.print_debug;

public abstract class AbstractDataSource implements DataListener {
	
	// **********************
	Vector<DataListener> listeners = new Vector<DataListener>();

	public synchronized void addDataChangeListener(DataListener d) {
		listeners.addElement(d);
		Object added = listeners.get(listeners.lastIndexOf(d));
		print_debug(this, added.getClass().getSimpleName() + " ("
				+ added.hashCode() + ") registered.");
	}

	public synchronized void removeDataChangeListener(DataListener d) {
		listeners.removeElement(d);
	}

	protected void notifyListeners(int eventid, DataEvent evt) {
		DataListener d;
		for (Enumeration<DataListener> e = listeners.elements(); e
				.hasMoreElements();) {
			d = e.nextElement();
			switch (eventid) {
			case DataEvent.DATA_CHANGED:
				d.dataChanged(evt);
				break;
			default:
				break;
			}
		}
		return;
	}

	/** {@inheritDoc} */
	public void dataChanged(DataEvent evt) {
		notifyListeners(DataEvent.DATA_CHANGED, evt);
	}
}
