/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import static abmland.common.TypesArith.reduceNA;
import static abmland.common.TypesArith.OP.MULT;
import static org.apache.log4j.Logger.getLogger;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static abmland.common.LogExceptionHandler.handleException;

import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.TypeVariable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import repast.simphony.random.RandomHelper;
import repast.simphony.space.grid.CellAccessor;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.space.grid.SingleOccupancyCellAccessor;
import repast.simphony.valueLayer.GridValueLayer;
import repast.simphony.valueLayer.ValueLayer;
import abmland.data.Businesstypes;
import abmland.data.CLCover;
import abmland.data.Householdtypes;
import abmland.data.Issuetypes;
import abmland.mainctrl.config.ConfManager;
import abmland.mainctrl.config.MiscConf;
import abmland.types.AbstractBoundedFloat;
import abmland.types.AbstractBoundedInteger;
import abmland.types.IBusinesstypesAllCells;
import abmland.types.ICLCoversAllCells;
import abmland.types.IHouseholdNumber;
import abmland.types.IHouseholdNumberPerTypeAllCells;
import abmland.types.INumber;
import abmland.types.IPressure;
import abmland.types.IPriceIndex;
import abmland.types.IProposalsSet;
import abmland.types.ISatisfaction;
import abmland.types.ISatisfactionForXTable;
import abmland.types.ISatisfactionPerIssueOfAgentAllCells;
import abmland.types.ISpace;
import abmland.types.ISupply;
import abmland.types.impl.AbstractAllCells;
import abmland.types.impl.BooleanAllCells;
import abmland.types.impl.BusinesstypesAllCells;
import abmland.types.impl.CLCoverAllCells;
import abmland.types.impl.CLCoverSomeCells;
import abmland.types.impl.CLCoversAllCells;
import abmland.types.impl.FloorSpaceAllCells;
import abmland.types.impl.HouseholdNumber;
import abmland.types.impl.HouseholdNumberAllCells;
import abmland.types.impl.HouseholdNumberPerTypeAllCells;
import abmland.types.impl.LandPriceIndexAllCells;
import abmland.types.impl.Pressure;
import abmland.types.impl.PriceIndex;
import abmland.types.impl.ProposalsSet;
import abmland.types.impl.Satisfaction;
import abmland.types.impl.SatisfactionForXTable;
import abmland.types.impl.SatisfactionPerIssueOfAgentAllCells;
import abmland.types.impl.Space;
import abmland.types.impl.Supply;
import abmland.types.impl.SupplyAllCells;

import com.google.common.annotations.VisibleForTesting;
import com.google.common.collect.Multimap;
import com.moseph.gis.raster.Raster;

/**
 * @author dkahlenb
 *
 */
public enum CellDumbMock {

	/** Singleton instance as Enum type */
	INSTANCE;

	private int[] areaDims;

	static boolean randomize() {
		return RandomHelper.nextInt() % 2 == 0 ? false : true;
	}

	static Double randomizeMinus1t1() {
		return RandomHelper.nextInt() % 2 == 0 ? new Double(Math.random())
				: new Double(-Math.random());
	}

	private CellDumbMock() {
		areaDims = ConfManager.MISCCONF.getConfig(MiscConf.class).areaParameters
				.getDims();
		RandomHelper.init();

	}

	private static final Logger log = getLogger(CellDumbMock.class);

	// static int[] dimensions = new int[] { 3, 3 };
	// private static String GRID = "Grid";
	// private static String CONTEXT = "Context";
	// private static String MAIN = "main";
	// private static Context<Object> main;

	// @VisibleForTesting
	// static void gridSetUp() {
	// RunState.init(null, null, null);
	// main = new DefaultContext<Object>(MAIN);
	// main.addProjection(GridFactoryFinder.createGridFactory(null)
	// .createGrid(
	// "AssayArea",
	// main,
	// new GridBuilderParameters<Object>(new StrictBorders(),
	// new RandomGridAdder<Object>(), false, 4, 4)));
	// SetMasterContext(main);
	// }

	// private static GridValueLayer valueLayer;

	// private static MiscConf config;
	//
	// private static int[] areaDims;

	// private static void layerSetUp() {
	// valueLayer = new GridValueLayer("Test ValueLayer", true,
	// new repast.simphony.space.grid.StickyBorders(),
	// GetSimulationDimensionsIntArray());
	// }

	// private static void gridSetUp0(String root) {
	// RunState.init(null, null, null);
	// main = new DefaultContext<Object>(root);
	// SetMasterContext(main);
	// }

	// private static <T> void gridSetHierarchy(Context<T> context) {
	//
	// Object id = context.getId();
	// main.addSubContext(context);
	// assertNotNull(FindContext("/" + MAIN + "/"));
	// assertNotNull(FindContext("/" + MAIN + "/").findContext(id));
	// assertNotNull(FindContext("/" + MAIN + "/").getSubContext(id));
	// assertNotNull(FindContext("/" + MAIN + "/" + context.getTypeID()));
	// assertEquals(context.getTypeID(), main.findContext(id).getTypeID());
	// }

	// /**
	// *
	// * @param <T>
	// * @param gridContext
	// * @param parent
	// * A `String` of the form `/sub1/sub2/`
	// */
	// private static <T> void gridSetHierarchy0(Context<T> gridContext,
	// String parent) {
	//
	// FindContext(parent).addSubContext(gridContext);
	// assertNotNull(FindContext(parent + gridContext.getTypeID()));
	// }

	// private static <T> void gridAssociateProjection(Context<T> context,
	// String gridName, boolean multi) {
	// int xDim = GetSimulationDimensionsIntArray()[0];
	// int yDim = GetSimulationDimensionsIntArray()[1];
	// Grid<T> grid = multi ? CreateMultiOccupancy2DGrid(gridName, context,
	// xDim, yDim) : CreateSingleOccupancy2DGrid(gridName, context,
	// xDim, yDim);
	// assertEquals(grid.getName(), context.getProjection(gridName).getName());
	// }

	// /**
	// * XXX Daniel One could introduce and use here - a method mutate - all
	// used
	// * data types have, and which is called on the value object, before it is
	// * added actually.
	// *
	// * @param <T>
	// * @param context
	// * @param value
	// */
	// private static <T> void gridAddObject(Context<T> context, T value) {
	// assertEquals(context, AddAgentToContext(
	// "/" + MAIN + "/" + context.getTypeID(), value).findContext(
	// context.getId()));
	// IndexedIterable<T> objectsInContext = context.getObjects(value
	// .getClass());
	// assertNotNull(objectsInContext);
	// for (T _type : objectsInContext) {
	// assertEquals(value.getClass(), _type.getClass());
	// }
	// }

	// static Random randomGenerator = new Random();

	/**
	 * @param i1
	 *            the minimum random value
	 * @param i2
	 *            the limiting random value (lowest upper value not generated)
	 * @return a pseudorandom, uniformly distributed `int` value between `i1`
	 *         (inclusive) and `i2` (exclusive), drawn from this random number
	 *         generator's sequence. The method used internally is implemented
	 *         by class `repast.simphony.random.RandomHelper`
	 */
	public static int nextIntFromTo(int i1, int i2) {

		return RandomHelper.nextIntFromTo(i1, i2 - 1);
	}

	/**
	 * @param fromI
	 *            the minimum random value
	 * @return a pseudorandom, uniformly distributed `int` value between `i1`
	 *         (inclusive) and the highest `int`, drawn from this random number
	 *         generator's sequence. The method used internally is implemented
	 *         by class `repast.simphony.random.RandomHelper`
	 */
	public static int nextIntFrom(int fromI) {
		return Math.max(fromI, RandomHelper.nextIntFromTo(fromI,
				Integer.MAX_VALUE));
	}

	public static <T> Collection<T> fill(int[] dimensions, T table) {

		List<T> pressures = new ArrayList<T>();
		int cells = reduceNA(MULT, dimensions);

		for (int i = 0; i < cells; i++) {
			assertTrue(pressures.add(table));
		}

		return pressures;
	}

	// private static <E extends Enum<E>> Collection<E> fill(int[] dimensions,
	// E enumValue) {
	//
	// Collection<E> enumColl = new ArrayList<E>();
	// int cells = 1;
	// for (int dim : dimensions) {
	// cells = cells * dim;
	// }
	//
	// for (int i = 0; i < cells; i++) {
	// assertTrue(enumColl.add(enumValue));
	// }
	//
	// return enumColl;
	// }

	/**
	 *
	 * @param <E>
	 *            An enum type
	 * @param dimensions
	 * @param eclazz
	 * @return A collection filled with random values of type `E`
	 */
	private static <E extends Enum<E>> Collection<E> fill(int[] dimensions,
			Class<E> eclazz) {

		int SIZE = EnumSet.allOf(eclazz).size();

		Iterator<E> eite = EnumSet.allOf(eclazz).iterator();
		int enumOrdinal = nextIntFromTo(1, SIZE);

		Collection<E> ecoll = new ArrayList<E>();
		int cells = 1;
		for (int dim : dimensions) {
			cells = cells * dim;
		}
		for (int i = 0; i < cells; i++) {

			// randomly choose enum constants
			enumOrdinal = nextIntFromTo(1, SIZE);
			eite = EnumSet.allOf(eclazz).iterator();
			E e = null;

			while (enumOrdinal-- > 1) {
				e = eite.next();
			}

			assertTrue(ecoll.add(e));
		}

		return ecoll;
	}

	private static <E extends Enum<E>> Collection<E> fill(Class<E> eclazz,
			GridPoint... points) {

		int SIZE = EnumSet.allOf(eclazz).size();

		Iterator<E> eite = EnumSet.allOf(eclazz).iterator();
		int enumOrdinal = nextIntFromTo(1, SIZE);

		Collection<E> ecoll = new ArrayList<E>();

		for (GridPoint g : points) {

			// randomly choose enum constants
			enumOrdinal = nextIntFromTo(1, SIZE);
			eite = EnumSet.allOf(eclazz).iterator();
			E e = null;

			while (enumOrdinal-- > 1) {
				e = eite.next();
			}

			assertTrue(ecoll.add(e));
		}

		return ecoll;
	}

	/**
	 *
	 * @param <E>
	 *            enum type
	 * @param dimensions
	 * @param earr
	 *            the initial array of type `E`, if you want randomised arrays
	 *            distributed in the collection, give a zero-length array here
	 *            (`new X[0]`)
	 * @return
	 */
	private static <E extends Enum<E>> Collection<E[]> fill(int[] dimensions,
			E[] earr) {
		Collection<E[]> earrcoll = new ArrayList<E[]>();
		int cells = 1;
		for (int dim : dimensions) {
			cells = cells * dim;
		}
		for (int i = 0; i < cells; i++) {
			assertTrue(earrcoll.add(0 != earr.length ? earr : CellDumbMock
					.randomize(earr)));
		}
		return earrcoll;
	}

	/**
	 * @param <E>
	 * @param e
	 *            One optional array of `CLCover` used for each Cell
	 * @return A collection of arrays of `CLCover`
	 */
	private static <E extends Enum<E>> Collection<E> fill(int[] dimensions, E e) {
		Collection<E> ecoll = new ArrayList<E>();
		int cells = 1;
		for (int dim : dimensions.length != 0 ? dimensions : INSTANCE.areaDims) {
			cells = cells * dim;
		}
		for (int i = 0; i < cells; i++) {
			assertTrue(ecoll.add(CellDumbMock.randomize(e)));
		}
		return ecoll;
	}

	private static <E extends Enum<E>> Collection<E> fill(E e,
			GridPoint... points) {
		Collection<E> ecoll = new ArrayList<E>();

		for (GridPoint p : points) {
			assertTrue(ecoll.add(e));
		}
		return ecoll;
	}

	private static <T> Collection<T> fill(int[] dimensions, T... f) {

		Collection<T> values = new ArrayList<T>();
		int cells = 1;
		for (int dim : dimensions) {
			cells = cells * dim;
		}

		switch (Clazzes.valueOf(f.getClass().isArray() ? f.getClass()
				.getComponentType().getSimpleName() : f.getClass()
				.getSimpleName())) {
		case ISpace:
			for (int i = 0; i < cells; i++) {
				assertTrue(values.add((T) (//
						f.length > 0 ? //
						f[0]//
								: new Space(nextIntFromTo(Space.MIN.intValue(),
										Space.MAX.intValue() + 1))//
						)));
			}
			break;
		case IHouseholdNumber:
			for (int i = 0; i < cells; i++) {
				assertTrue(values.add((T) (//
						f.length > 0 ? //
						f[0]//
								: new HouseholdNumber(nextIntFromTo(
										HouseholdNumber.MIN.intValue(),
										HouseholdNumber.MAX.intValue() + 1))//
						)));
			}
			break;
		case IPriceIndex:
			for (int i = 0; i < cells; i++) {
				assertTrue(values.add((T) (//
						f.length > 0 ? //
						f[0]//
								: new PriceIndex(nextIntFromTo(PriceIndex.MIN
										.intValue(),
										PriceIndex.MAX.intValue() + 1))//
						)));
			}
			break;
		case ISupply:
			for (int i = 0; i < cells; i++) {
				assertTrue(values.add((T) (//
						f.length > 0 ? //
						f[0]//
								: new Supply(nextIntFromTo(Supply.MIN
										.intValue(), Supply.MAX.intValue() + 1))//
						)));
			}
			break;
		case Boolean:
			for (int i = 0; i < cells; i++) {
				assertTrue(values.add((T) (//
						f.length > 0 ? //
						f[0]//
								: new Boolean(
										RandomHelper.nextInt() % 2 == 0 ? false
												: true)//
						)));
			}
			break;
		default:
			break;
		}

		return values;
	}

	private static Collection<Boolean> fill(int[] dimensions, boolean... bool) {

		List<Boolean> booleans = new ArrayList<Boolean>();

		int cells = 1;
		for (int dim : null != dimensions ? dimensions : INSTANCE.areaDims) {
			cells = cells * dim;
		}
		for (int i = 0; i < cells; i++) {
			/*
			 * assert the collection really changed - respective the element is
			 * added. Depends on the implementation, i.e. HashSet functionally
			 * differs here from ArrayList
			 */
			assertTrue(booleans.add(bool.length > 0 ? bool[0] : CellDumbMock
					.randomize()));
		}
		assertEquals(cells, booleans.toArray().length);
		return booleans;
	}

	/**
	 *
	 * @param &lt;E extends Enum&lt;E&gt;&gt;
	 * @param emptyEnumArray
	 * @tag usage.parameter : Give i. e. `new Businesstypes[0]` (where
	 *      `Businesstypes` has type `Enum&lt;E&gt;`)
	 * @return
	 */
	private static <E extends Enum<E>> E[] randomize(E[] emptyEnumArray) {

		// This (.getComponentType()) would return null if
		Class<E> enumType = (Class<E>) emptyEnumArray.getClass()
				.getComponentType();

		final int ENUMCOUNT = EnumSet.allOf(enumType).size();

		List<E> enumList = new ArrayList<E>();
		ListIterator<E> iterableOnEnumList = enumList.listIterator();

		for (int i = 1; i <= nextIntFromTo(1, ENUMCOUNT + 1); i++) {
			iterableOnEnumList.add(EnumSet.allOf(enumType).toArray(
					emptyEnumArray)[nextIntFromTo(1, ENUMCOUNT)]);
		}

		Set<E> shuffledEnumList = new HashSet<E>();
		shuffledEnumList.addAll(enumList);

		return shuffledEnumList.toArray(emptyEnumArray);
	}

	/**
	 * For a given list this returns the upper left or bottom right corner,
	 * meaning: ul:: GridPoint(MIN, _), i. e. (0, 100) for x_MIN = 0
	 *
	 * br :: GridPoint(_, MAX), i. e. (1, 100) for y_MAX = 100
	 *
	 * problem : GridPoint doesn't implement Comparable
	 *
	 * @param gplist
	 * @return
	 */
	static List<GridPoint> sort(List<GridPoint> gplist) {
		return gplist;
	}

	static <E extends Enum<E>> E randomize(E e) {
		int SIZE = e.getClass().getEnumConstants().length;

		E random = (E) e.getClass().getEnumConstants()[nextIntFromTo(1, SIZE)];
		return random;
	}

	private enum Clazzes {
		IPriceIndex, Boolean, IHouseholdNumber, ISupply, ISpace //
		, IPressure, ISatisfaction, MapKeysAreGridPoint;
	}

	protected abstract interface AnAllCells<T> {

		/**
		 *
		 *
		 * @param <T>
		 *            The type to be stored in the AllCells object
		 * @param layerName
		 *            The name for the returned grid-value layer
		 * @tag usage.general : For `dimensions` we use configuration driven
		 *      default dimensions. The dimensions (two supported) the
		 *      grid-value layer should have.
		 * @return A grid-value layer filled with randomly selected/generated
		 *         values of type `T`
		 */
		abstract AbstractAllCells<T> requestAllCells(String layerName);

		/**
		 * @param <T>
		 *            The type to be stored in the AllCells object
		 * @param layerName
		 *            The name for the returned grid-value layer
		 * @param value
		 *            A value of type `T` to be stored into each 'cell'
		 * @tag usage.general : For `dimensions` we use configuration driven
		 *      default dimensions. The dimensions (two supported) the
		 *      grid-value layer should have.
		 * @return A grid-value layer filled overall with the given `T`-typed
		 *         value
		 */
		abstract AbstractAllCells<T> requestAllCells(String layerName, T value);

		/**
		 * @param <T>
		 *            The type to be stored in the AllCells object
		 * @param values
		 *            A collection of `T`-typed values to be layer-stored
		 * @param layerName
		 *            A name for the returned grid-value layer
		 * @tag usage.general : For `dimensions` we use configuration driven
		 *      default dimensions. The dimensions (two supported) the
		 *      grid-value layer should have.
		 * @return A grid-value layer containing the values from `supplies`
		 */
		abstract AbstractAllCells<T> requestAllCells(Collection<T> values,
				String layerName);

		/**
		 * @param <T>
		 *            The type to be stored in the AllCells object
		 * @param values
		 *            A map of `T`-typed values mapped to grid points
		 * @param layerName
		 *            The name for the returned grid-value layer
		 * @tag usage.general : For `dimensions` we use configuration driven
		 *      default dimensions. The dimensions (two supported) the
		 *      grid-value layer should have.
		 * @return A grid-value layer containing the values from the given map
		 */
		abstract AbstractAllCells<T> requestAllCells(Map<GridPoint, T> values,
				String layerName);

		/**
		 * @param <T>
		 *            The type to be stored in the AllCells object
		 * @param values
		 *            A raster containing double equivalents for to be `T`-typed
		 *            values
		 * @param layerName
		 *            The name for the returned grid-value layer
		 * @tag usage.general : For `dimensions` we use configuration driven
		 *      default dimensions. The dimensions (two supported) the
		 *      grid-value layer should have.
		 * @return A grid-value layer containing the now `T`-typed values from
		 *         the given raster
		 */
		abstract AbstractAllCells<T> requestAllCells(Raster values,
				String layerName);
	}

	public static class ABooleanAllCells implements AnAllCells<Boolean> { // OK

		/**
		 * @see AnAllCells#requestAllCells(String, int[])
		 */
		public static BooleanAllCells requestBooleanAllCells(String layerName) {

			return (BooleanAllCells) new ABooleanAllCells()
					.requestAllCells(layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(String, Object, int[])
		 */
		public static BooleanAllCells requestBooleanAllCells(String layerName,
				Boolean bool) {

			return (BooleanAllCells) new ABooleanAllCells().requestAllCells(
					layerName, bool);
		}

		/**
		 * @see AnAllCells#requestAllCells(Collection, String, int[])
		 */
		public static BooleanAllCells requestBooleanAllCells(
				Collection<Boolean> booleans, String layerName) {

			return (BooleanAllCells) new ABooleanAllCells().requestAllCells(
					booleans, layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(Map, String, int[])
		 */
		public static BooleanAllCells requestBooleanAllCells(
				Map<GridPoint, Boolean> booleans, String layerName) {

			return (BooleanAllCells) new ABooleanAllCells().requestAllCells(
					booleans, layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(Raster, String, int[])
		 */
		public static BooleanAllCells requestBooleanAllCells(Raster booleans,
				String layerName) {

			return (BooleanAllCells) new ABooleanAllCells().requestAllCells(
					booleans, layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<Boolean> requestAllCells(String layerName) {

			return requestAllCells(CellDumbMock.fill(INSTANCE.areaDims,
					new Boolean[0]), layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<Boolean> requestAllCells(String layerName,
				Boolean value) {

			return requestAllCells(CellDumbMock.fill(INSTANCE.areaDims, value),
					layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<Boolean> requestAllCells(
				Collection<Boolean> values, String layerName) {

			return requestFullLayer(values, new BooleanAllCells(
					getDefaultGVLayer(layerName, null)));
		}

		/** {@inheritDoc} */
		public AbstractAllCells<Boolean> requestAllCells(
				Map<GridPoint, Boolean> values, String layerName) {

			return requestFullLayer(values, new BooleanAllCells(
					getDefaultGVLayer(layerName, null)));
		}

		/** {@inheritDoc} */
		public AbstractAllCells<Boolean> requestAllCells(Raster values,
				String layerName) {

			return requestFullLayer(values, new BooleanAllCells(
					getDefaultGVLayer(layerName, null)));
		}
	}

	public static class ACLCoverAllCells implements AnAllCells<CLCover> { // OK

		/**
		 * @see AnAllCells#requestAllCells(String, int[])
		 */
		public static CLCoverAllCells requestCLCoverAllCells(String layerName) {

			return (CLCoverAllCells) new ACLCoverAllCells()
					.requestAllCells(layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(String, Object, int[])
		 */
		public static CLCoverAllCells requestCLCoverAllCells(String layerName,
				CLCover clc) {

			return (CLCoverAllCells) new ACLCoverAllCells().requestAllCells(
					layerName, clc);
		}

		/**
		 * @see AnAllCells#requestAllCells(String, Object, int[])
		 */
		public static CLCoverAllCells requestCLCoverAllCells(String layerName,
				double clc) {

			return (CLCoverAllCells) new ACLCoverAllCells().requestAllCells(
					layerName, CLCover.valueOf(clc));
		}

		/**
		 * @see AnAllCells#requestAllCells(String, Object, int[])
		 */
		public static CLCoverAllCells requestCLCoverAllCells(String layerName,
				String clc) {

			return (CLCoverAllCells) new ACLCoverAllCells().requestAllCells(
					layerName, CLCover.valueOf(clc));
		}

		/**
		 * @see AnAllCells#requestAllCells(Collection, String, int[])
		 */
		public static CLCoverAllCells requestCLCoverAllCells(
				Collection<CLCover> clCovers, String layerName) {

			return (CLCoverAllCells) new ACLCoverAllCells().requestAllCells(
					clCovers, layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(Map, String, int[])
		 */
		public static CLCoverAllCells requestCLCoverAllCells(
				Map<GridPoint, CLCover> clCovers, String layerName) {

			return (CLCoverAllCells) new ACLCoverAllCells().requestAllCells(
					clCovers, layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(Raster, String, int[])
		 */
		public static CLCoverAllCells requestCLCoverAllCells(Raster clCovers,
				String layerName) {

			return (CLCoverAllCells) new ACLCoverAllCells().requestAllCells(
					clCovers, layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<CLCover> requestAllCells(String layerName) {

			return requestAllCells(CellDumbMock.fill(INSTANCE.areaDims,
					CLCover.class), layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<CLCover> requestAllCells(String layerName,
				CLCover value) {

			return requestAllCells(CellDumbMock.fill(INSTANCE.areaDims, value),
					layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<CLCover> requestAllCells(
				Collection<CLCover> values, String layerName) {

			return requestFullLayer(values, new CLCoverAllCells(
					getDefaultGVLayer(layerName, null)));
		}

		/** {@inheritDoc} */
		public AbstractAllCells<CLCover> requestAllCells(
				Map<GridPoint, CLCover> values, String layerName) {

			return requestFullLayer(values, new CLCoverAllCells(
					getDefaultGVLayer(layerName, null)));
		}

		/** {@inheritDoc} */
		public AbstractAllCells<CLCover> requestAllCells(Raster values,
				String layerName) {

			return requestFullLayer(values, new CLCoverAllCells(
					getDefaultGVLayer(layerName, null)));
		}
	}

	public static class ABusinesstypesAllCells { // OK

		/**
		 * @param dimension
		 * @tag usage.parameter : Give `null` here as `dimension` to have
		 *      configuration driven dimensions
		 * @param businesstypes
		 * @tag usage.parameter : Give `new Businesstypes[0]` (empty array) here
		 *      to have random arrays at the grid points
		 * @return A gridpoint-map containing the given `businesstypes` array at
		 *         each entry or randomly selected `Businesstypes[]` entries if
		 *         `businesstypes` was empty
		 */
		public static IBusinesstypesAllCells requestBusinesstypesAllCells(
				Businesstypes[] businesstypes) {

			return requestBusinesstypesAllCells(CellDumbMock.fill(
					INSTANCE.areaDims, businesstypes));
		}

		/**
		 * @param dimension
		 * @tag usage.parameter : Give `null` here as `dimension` to have
		 *      configuration driven dimensions.
		 * @param businesstypes
		 *            A collection of arrays of `Businesstypes` to be copied
		 * @return A gridpoint-map containing the values from the collection
		 */
		public static IBusinesstypesAllCells requestBusinesstypesAllCells(
				Collection<Businesstypes[]> businesstypes) {

			return requestMultimapToGridPoint(businesstypes,
					new BusinesstypesAllCells(), INSTANCE.areaDims);
		}
	}

	public static class ACLCoversAllCells { // OK

		/**
		 * @param dimension
		 * @tag usage.parameter : Give `null` here as `dimension` to have
		 *      configuration driven dimensions
		 * @param clCovers
		 * @tag usage.parameter : Give `new CLCover[0]` (empty array) here to
		 *      have random arrays at the grid points
		 * @return A gridpoint-map containing the given `clCovers` array at each
		 *         entry or randomly selected `CLCover[]` entries if clCovers
		 *         was empty
		 */
		public static ICLCoversAllCells requestCLCoversAllCells(
				CLCover[] clCovers) {

			return requestCLCoversAllCells(CellDumbMock.fill(INSTANCE.areaDims,
					clCovers));
		}

		/**
		 * @param dimension
		 * @tag usage.parameter : Give `null` here as `dimension` to have
		 *      configuration driven dimensions.
		 * @param clCovers
		 *            the collection to be copied.
		 * @return A gridpoint-map containing the values from the collection
		 */
		public static ICLCoversAllCells requestCLCoversAllCells(
				Collection<CLCover[]> clCovers) {

			return requestMultimapToGridPoint(clCovers, new CLCoversAllCells(),
					INSTANCE.areaDims);
		}
	}

	public static class ACLCoverSomeCells { // TODO [1] ADD
		// request...(Map<GridPoint,...) to
		// pass a gridpoint associated map

		/**
		 * @param dimensions
		 * @tag usage.parameter : Give an empty array (`new int[0]`) here to
		 *      have the default dimensions
		 * @return A gridpointmap filled with randomly selected `CLCover` values
		 */
		public static Map<GridPoint, CLCover> requestCLCoverSomeCells(
				GridPoint... points) {
			return requestCLCoverSomeCells(fill(CLCover.class, points), points);
		}

		// /**
		// * @param clCover
		// * A `CLCover` value
		// * @return A gridpointmap filled with the `clCover` value
		// */
		// public static Map<GridPoint, CLCover> requestCLCoverSomeCells(
		// int[] dimensions, CLCover clCover) {
		// return requestCLCoverSomeCells(fill(dimensions, clCover));
		// }
		//
		// /**
		// * @param clCover
		// * A `double` value equiv. to the `CLCover`
		// * @return A gridpointmap filled with the `clCover` value
		// */
		// public static Map<GridPoint, CLCover> requestCLCoverSomeCells(
		// int[] dimensions, double clCover) {
		// return requestCLCoverSomeCells(fill(dimensions, CLCover
		// .valueOf(clCover)));
		// }

		/**
		 *
		 * @param clCover
		 *            A `Collection` of `CLCover`-typed values
		 * @return A gridpointmap filled with the `clCover` values
		 */
		public static Map<GridPoint, CLCover> requestCLCoverSomeCells(
				Collection<CLCover> clCover, GridPoint... points) {
			return requestMapFromCollection(clCover, new CLCoverSomeCells(
					new HashMap<GridPoint, CLCover>()), points);
		}
	}

	public static class AFloorSpaceAllCells implements AnAllCells<ISpace> { // OK

		/**
		 * @see AnAllCells#requestAllCells(String, int[])
		 */
		public static FloorSpaceAllCells requestFloorSpaceAllCells(
				String layerName) {

			return (FloorSpaceAllCells) new AFloorSpaceAllCells()
					.requestAllCells(layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(String, Object, int[])
		 */
		public static FloorSpaceAllCells requestFloorSpaceAllCells(
				String layerName, ISpace space) {

			return (FloorSpaceAllCells) new AFloorSpaceAllCells()
					.requestAllCells(layerName, space);
		}

		/**
		 * @see AnAllCells#requestAllCells(String, Object, int[])
		 */
		public static FloorSpaceAllCells requestFloorSpaceAllCells(
				String layerName, int space) {

			return (FloorSpaceAllCells) new AFloorSpaceAllCells()
					.requestAllCells(layerName, new Space(space));
		}

		/**
		 * @see AnAllCells#requestAllCells(Collection, String, int[])
		 */
		public static FloorSpaceAllCells requestFloorSpaceAllCells(
				Collection<ISpace> spaces, String layerName) {

			return (FloorSpaceAllCells) new AFloorSpaceAllCells()
					.requestAllCells(spaces, layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(Map, String, int[])
		 */
		public static FloorSpaceAllCells requestFloorSpaceAllCells(
				Map<GridPoint, ISpace> spaces, String layerName) {

			return (FloorSpaceAllCells) new AFloorSpaceAllCells()
					.requestAllCells(spaces, layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(Raster, String, int[])
		 */
		public static FloorSpaceAllCells requestFloorSpaceAllCells(
				Raster spaces, String layerName) {

			return (FloorSpaceAllCells) new AFloorSpaceAllCells()
					.requestAllCells(spaces, layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<ISpace> requestAllCells(String layerName) {

			return requestAllCells(CellDumbMock.fill(INSTANCE.areaDims,
					new ISpace[0]), layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<ISpace> requestAllCells(String layerName,
				ISpace value) {

			return requestAllCells(CellDumbMock.fill(INSTANCE.areaDims, value),
					layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<ISpace> requestAllCells(
				Collection<ISpace> values, String layerName) {

			return requestFullLayer(values, new FloorSpaceAllCells(
					getDefaultGVLayer(layerName, null)));
		}

		/** {@inheritDoc} */
		public AbstractAllCells<ISpace> requestAllCells(
				Map<GridPoint, ISpace> values, String layerName) {

			return requestFullLayer(values, new FloorSpaceAllCells(
					getDefaultGVLayer(layerName, null)));
		}

		/** {@inheritDoc} */
		public AbstractAllCells<ISpace> requestAllCells(Raster values,
				String layerName) {

			return requestFullLayer(values, new FloorSpaceAllCells(
					getDefaultGVLayer(layerName, null)));
		}
	}

	public static class AHouseholdNumberAllCells implements
			AnAllCells<IHouseholdNumber> { // OK

		/**
		 * @see AnAllCells#requestAllCells(String, int[])
		 */
		public static HouseholdNumberAllCells requestHouseholdNumberAllCells(
				String layerName) {

			return (HouseholdNumberAllCells) new AHouseholdNumberAllCells()
					.requestAllCells(layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(String, Object, int[])
		 */
		public static HouseholdNumberAllCells requestHouseholdNumberAllCells(
				String layerName, IHouseholdNumber number) {

			return (HouseholdNumberAllCells) new AHouseholdNumberAllCells()
					.requestAllCells(layerName, number);
		}

		/**
		 * @see AnAllCells#requestAllCells(String, Object, int[])
		 */
		public static HouseholdNumberAllCells requestHouseholdNumberAllCells(
				String layerName, int number) {

			return (HouseholdNumberAllCells) new AHouseholdNumberAllCells()
					.requestAllCells(layerName, new HouseholdNumber(number));
		}

		/**
		 * @see AnAllCells#requestAllCells(Collection, String, int[])
		 */
		public static HouseholdNumberAllCells requestHouseholdNumberAllCells(
				Collection<IHouseholdNumber> householdNumbers, String layerName) {

			return (HouseholdNumberAllCells) new AHouseholdNumberAllCells()
					.requestAllCells(householdNumbers, layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(Map, String, int[])
		 */
		public static HouseholdNumberAllCells requestHouseholdNumberAllCells(
				Map<GridPoint, IHouseholdNumber> householdNumbers,
				String layerName) {

			return (HouseholdNumberAllCells) new AHouseholdNumberAllCells()
					.requestAllCells(householdNumbers, layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(Raster, String, int[])
		 */
		public static HouseholdNumberAllCells requestHouseholdNumberAllCells(
				Raster householdNumbers, String layerName) {

			return (HouseholdNumberAllCells) new AHouseholdNumberAllCells()
					.requestAllCells(householdNumbers, layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<IHouseholdNumber> requestAllCells(
				String layerName) {

			return requestAllCells(CellDumbMock.fill(INSTANCE.areaDims,
					new IHouseholdNumber[0]), layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<IHouseholdNumber> requestAllCells(
				String layerName, IHouseholdNumber value) {

			return requestAllCells(CellDumbMock.fill(INSTANCE.areaDims, value),
					layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<IHouseholdNumber> requestAllCells(
				Collection<IHouseholdNumber> values, String layerName) {

			return requestFullLayer(values, new HouseholdNumberAllCells(
					getDefaultGVLayer(layerName, null)));
		}

		/** {@inheritDoc} */
		public AbstractAllCells<IHouseholdNumber> requestAllCells(
				Map<GridPoint, IHouseholdNumber> values, String layerName) {

			return requestFullLayer(values, new HouseholdNumberAllCells(
					getDefaultGVLayer(layerName, null)));
		}

		/** {@inheritDoc} */
		public AbstractAllCells<IHouseholdNumber> requestAllCells(
				Raster values, String layerName) {

			return requestFullLayer(values, new HouseholdNumberAllCells(
					getDefaultGVLayer(layerName, null)));
		}
	}

	public static class ASupplyAllCells implements AnAllCells<ISupply> { // OK

		/**
		 * @see AnAllCells#requestAllCells(String, int[])
		 */
		public static SupplyAllCells requestSupplyAllCells(String layerName) {

			return (SupplyAllCells) new ASupplyAllCells()
					.requestAllCells(layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(String, Object, int[])
		 */
		public static SupplyAllCells requestSupplyAllCells(String layerName,
				ISupply supply) {

			return (SupplyAllCells) new ASupplyAllCells().requestAllCells(
					layerName, supply);
		}

		/**
		 * @see AnAllCells#requestAllCells(String, Object, int[])
		 */
		public static SupplyAllCells requestSupplyAllCells(String layerName,
				float supply) {

			return (SupplyAllCells) new ASupplyAllCells().requestAllCells(
					layerName, new Supply(supply));
		}

		/**
		 * @see AnAllCells#requestAllCells(Collection, String, int[])
		 */
		public static SupplyAllCells requestSupplyAllCells(
				Collection<ISupply> supplies, String layerName) {

			return (SupplyAllCells) new ASupplyAllCells().requestAllCells(
					supplies, layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(Map, String, int[])
		 */
		public static SupplyAllCells requestSupplyAllCells(
				Map<GridPoint, ISupply> supplies, String layerName) {

			return (SupplyAllCells) new ASupplyAllCells().requestAllCells(
					supplies, layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(Raster, String, int[])
		 */
		public static SupplyAllCells requestSupplyAllCells(Raster supplies,
				String layerName) {

			return (SupplyAllCells) new ASupplyAllCells().requestAllCells(
					supplies, layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<ISupply> requestAllCells(String layerName) {

			return requestAllCells(CellDumbMock.fill(INSTANCE.areaDims,
					new ISupply[0]), layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<ISupply> requestAllCells(String layerName,
				ISupply value) {

			return requestAllCells(CellDumbMock.fill(INSTANCE.areaDims, value),
					layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<ISupply> requestAllCells(
				Collection<ISupply> values, String layerName) {

			return requestFullLayer(values, new SupplyAllCells(
					getDefaultGVLayer(layerName, null)));
		}

		/** {@inheritDoc} */
		public AbstractAllCells<ISupply> requestAllCells(
				Map<GridPoint, ISupply> values, String layerName) {

			return requestFullLayer(values, new SupplyAllCells(
					getDefaultGVLayer(layerName, null)));
		}

		/** {@inheritDoc} */
		public AbstractAllCells<ISupply> requestAllCells(Raster values,
				String layerName) {

			return requestFullLayer(values, new SupplyAllCells(
					getDefaultGVLayer(layerName, null)));
		}
	}

	public static class ALandPriceIndexAllCells implements
			AnAllCells<IPriceIndex> { // OK

		/**
		 * @see AnAllCells#requestAllCells(String, int[])
		 */
		public static LandPriceIndexAllCells requestLandPriceIndexAllCells(
				String layerName) {

			return (LandPriceIndexAllCells) new ALandPriceIndexAllCells()
					.requestAllCells(layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(String, Object, int[])
		 */
		public static LandPriceIndexAllCells requestLandPriceIndexAllCells(
				String layerName, IPriceIndex price) {

			return (LandPriceIndexAllCells) new ALandPriceIndexAllCells()
					.requestAllCells(layerName, price);
		}

		/**
		 * @see AnAllCells#requestAllCells(String, Object, int[])
		 */
		public static LandPriceIndexAllCells requestLandPriceIndexAllCells(
				String layerName, float price) {

			return (LandPriceIndexAllCells) new ALandPriceIndexAllCells()
					.requestAllCells(layerName, new PriceIndex(price));
		}

		/**
		 * @see AnAllCells#requestAllCells(Collection, String, int[])
		 */
		public static LandPriceIndexAllCells requestLandPriceIndexAllCells(
				Collection<IPriceIndex> priceIndex, String layerName) {

			return (LandPriceIndexAllCells) new ALandPriceIndexAllCells()
					.requestAllCells(priceIndex, layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(Map, String, int[])
		 */
		public static LandPriceIndexAllCells requestLandPriceIndexAllCells(
				Map<GridPoint, IPriceIndex> priceIndex, String layerName) {

			return (LandPriceIndexAllCells) new ALandPriceIndexAllCells()
					.requestAllCells(priceIndex, layerName);
		}

		/**
		 * @see AnAllCells#requestAllCells(Raster, String, int[])
		 */
		public static LandPriceIndexAllCells requestLandPriceIndexAllCells(
				Raster priceIndex, String layerName) {

			return (LandPriceIndexAllCells) new ALandPriceIndexAllCells()
					.requestAllCells(priceIndex, layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<IPriceIndex> requestAllCells(String layerName) {

			return requestAllCells(CellDumbMock.fill(INSTANCE.areaDims,
					new IPriceIndex[0]), layerName);
		}

		/** {@inheritDoc} */
		public AbstractAllCells<IPriceIndex> requestAllCells(String layerName,
				IPriceIndex value) {

			return requestAllCells(CellDumbMock.fill(INSTANCE.areaDims, value),
					layerName);
		}

		/** {@inheritedDoc} */
		public AbstractAllCells<IPriceIndex> requestAllCells(
				Collection<IPriceIndex> values, String layerName) {

			return requestFullLayer(values, new LandPriceIndexAllCells(
					getDefaultGVLayer(layerName, null)));
		}

		/** {@inheritDoc} */
		public AbstractAllCells<IPriceIndex> requestAllCells(
				Map<GridPoint, IPriceIndex> values, String layerName) {

			return requestFullLayer(values, new LandPriceIndexAllCells(
					getDefaultGVLayer(layerName, null)));
		}

		/** {@inheritDoc} */
		public AbstractAllCells<IPriceIndex> requestAllCells(Raster values,
				String layerName) {

			// here is the last chance to look for the NODATA entries in the
			// raster and to construct the ranged typed value for this
			// peripheric point
			// in the fine manner (new PriceIndex(value, true)) if NODATA
			// TODO How to avoid the concrete implementation dependency here
			// instead using something like IPriceIndex.peripheric

			return requestFullLayer(values, new LandPriceIndexAllCells(
					getDefaultGVLayer(layerName, null)), PriceIndex
					.getInstance(Double.NaN, true));
		}
	}

	public static class AHouseholdNumberPerTypeAllCells { // OK

		private static Collection<Map<Householdtypes, IHouseholdNumber>> fill(
				Map<Householdtypes, IHouseholdNumber> mapping) {
			Collection<Map<Householdtypes, IHouseholdNumber>> numbers = new ArrayList<Map<Householdtypes, IHouseholdNumber>>();

			int cells = reduceNA(MULT, INSTANCE.areaDims);

			if (null != mapping) {
				for (int i = 0; i < cells; i++) {
					assertTrue(numbers.add(mapping));
				}
			} else {
				for (int i = 0; i < cells; i++) {
					Map<Householdtypes, IHouseholdNumber> _numbers = new HashMap<Householdtypes, IHouseholdNumber>();
					for (Householdtypes householdtype : Householdtypes.values()) {
						_numbers.put(householdtype, new HouseholdNumber(Math
								.round(Math.random() * 5e2)));
					}
					assertTrue(numbers.add(_numbers));
				}
			}
			return numbers;
		}

		/**
		 * @return A gridpoint map filled with randomly selected mappings
		 */
		public static IHouseholdNumberPerTypeAllCells requestHouseholdNumberPerTypeAllCells() {
			return requestHouseholdNumberPerTypeAllCells(fill(null));
		}

		/**
		 * @param mapping
		 *            A Map containing `Householdtypes` &lt;- `IHouseholdNumber`
		 *            mappings
		 * @return A gridpoint map filled with the given `mapping` value
		 */
		public static IHouseholdNumberPerTypeAllCells requestHouseholdNumberPerTypeAllCells(
				Map<Householdtypes, IHouseholdNumber> mapping) {
			return requestHouseholdNumberPerTypeAllCells(fill(mapping));
		}

		/**
		 * @param maps
		 *            A `Collection` of mappings
		 * @return A gridpoint-map filled with the values from the given
		 *         `Collection` maps
		 */
		public static IHouseholdNumberPerTypeAllCells requestHouseholdNumberPerTypeAllCells(
				Collection<Map<Householdtypes, IHouseholdNumber>> maps) {

			return requestMapToGridPoint(
					maps,
					new HouseholdNumberPerTypeAllCells(
							new HashMap<GridPoint, Map<Householdtypes, IHouseholdNumber>>()),
					INSTANCE.areaDims);
		}
	}

	public static class APermissionForProposalTable {

		/**
		 * @param proposals
		 *            The proposals given (optional). Used as keys
		 * @return filled map with random booleans as values mapped to the given
		 *         `proposals` values or mapped to three generated values if no
		 *         `proposals` parameter given
		 */
		public static Map<Map<GridPoint, CLCover>, Boolean> requestPermissionForProposalTable(
				List<Map<GridPoint, CLCover>> proposals) {

			return (Map<Map<GridPoint, CLCover>, Boolean>) (null != proposals
					&& proposals.size() > 0 //
			? requestTForXTable(proposals, Clazzes.Boolean) // else
					: requestTForXTable(randomize1(), Clazzes.Boolean));
		}

		/**
		 *
		 * @param bool
		 *            One value put to each key
		 * @param proposals
		 *            The proposals given. Used as keys
		 * @return filled map with `bool` value mapped to the given `proposals`
		 *         values or mapped to three generated values if no `proposals`
		 *         parameter given
		 */
		public static Map<Map<GridPoint, CLCover>, Boolean> requestPermissionForProposalTable(
				Boolean bool, List<Map<GridPoint, CLCover>> proposals) {
			return requestTForXTable(
					null != proposals && proposals.size() > 0 ? proposals
							: randomize1(), bool);
		}

		/**
		 * @param oldpermissions
		 *            A map of proposals given.
		 * @param bool
		 *            An optional boolean value
		 * @return Re-filled map with random booleans as values if no `bool`
		 *         parameter given or with `bool` as value
		 */
		@SuppressWarnings("unchecked")
		public static Map<Map<GridPoint, CLCover>, Boolean> requestPermissionForProposalTable(
				Map<Map<GridPoint, CLCover>, Boolean> oldpermissions,
				boolean... bool) {

			// make a list from the old permissions first
			List<Map<GridPoint, CLCover>> proposals = new ArrayList<Map<GridPoint, CLCover>>();
			Iterator<Map<GridPoint, CLCover>> _ = oldpermissions.keySet()
					.iterator();
			while (_.hasNext()) {
				proposals.add(_.next());
			}

			// return new PermissionForProposalTable(map);
			return (Map<Map<GridPoint, CLCover>, Boolean>) //
			(null != bool && bool.length > 0 // <- when bool value given
			? requestTForXTable(null != proposals && proposals.size() > 0 // &&
			// <- when proposals given
			? proposals // else with random proposals ->
					: randomize1(), bool[0]) // else with marker for random
					// boolean generator ->
					: requestTForXTable(null != proposals
							&& proposals.size() > 0 // && <- this case too when
					// proposals given
					? proposals // else with random proposals ->
							: randomize1(), Clazzes.Boolean));
		}
	}

	public static class APressureForCLCoverAllCells { // OK

		private static Collection<Map<CLCover, ?>> fill(Map<CLCover, ?> table) {

			List<Map<CLCover, ?>> pressures = new ArrayList<Map<CLCover, ?>>();
			int cells = reduceNA(MULT, INSTANCE.areaDims);

			if (null != table) {
				for (int i = 0; i < cells; i++) {
					assertTrue(pressures.add(table));
				}
			} else {
				for (int i = 0; i < cells; i++) {
					assertTrue(pressures.add(requestTForXTable(
							CLCover.values(), IPressure.class)));
				}
			}
			return pressures;
		}

		// /**
		// * @return A gridpoint-map filled with randomly selected
		// * `IPressureForXTable&lt;CLCover&gt;` values
		// */
		// public static IPressureForCLCoverAllCells
		// requestPressureForCLCoverAllCells() {
		// return requestPressureForCLCoverAllCells(CellDumbMock.fill(
		// INSTANCE.areaDims, requestTForXTable(CLCover.values(),
		// IPressure.class)));
		// }
		//
		// /**
		// * @param pressureTable
		// * A mapping
		// * @return A gridpoint-map filled with the given `pressureTable`
		// mapping
		// */
		// public static IPressureForCLCoverAllCells
		// requestPressureForCLCoverAllCells(
		// Collection<ItForXTable<CLCover, IPressure>> pressureTable) {
		// return requestPressureForCLCoverAllCells(fill(pressureTable),
		// INSTANCE.areaDims);
		// }
		//
		// /**
		// * @param pressures
		// * A collection of pressure values, one for each cell
		// * @return A gridpoint-map filled with the given pressure values
		// */
		// public static IPressureForCLCoverAllCells
		// requestPressureForCLCoverAllCells(
		// Collection<ItForXTable<CLCover, IPressure>> pressures) {
		//
		// return requestMapToGridPoint(pressures,
		// new PressureForCLCoverAllCells(
		// new HashMap<GridPoint, Map<CLCover, IPressure>>()),
		// INSTANCE.areaDims);
		// }
	}

	public static class APressureForCLCoverTable {

		/**
		 * @param clCovers
		 *            An optional array of `CLCover`
		 * @return A mapping of randomly selected `IPressure` values to the
		 *         given or all known `CLCover` values
		 */
		public static Map<CLCover, IPressure> requestPressureForCLCoverTable(
				CLCover... clCovers) {
			return (Map<CLCover, IPressure>) requestTForXTable(clCovers,
					Clazzes.IPressure);
		}

		/**
		 * @param pressure
		 *            An `IPressure` value
		 * @param clCovers
		 *            An optional array of `CLCover`
		 * @return A mapping of the given `IPressure` to the given or all known
		 *         `CLCover` values
		 */
		public static Map<CLCover, IPressure> requestPressureForCLCoverTable(
				IPressure pressure, CLCover... clCovers) {
			return requestTForXTable(clCovers, pressure);
		}

		/**
		 * @param pressure
		 *            A `Float` value
		 * @param clCovers
		 *            An optional array of `CLCover`
		 * @return A mapping of the given `IPressure` (from `Float`) to the
		 *         given or all known `CLCover` values
		 */
		public static Map<CLCover, Pressure> requestPressureForCLCoverTable(
				Float pressure, CLCover... clCovers) {
			return requestTForXTable(clCovers, new Pressure(pressure));
		}
	}

	public static class ASatisfactionForIssuesTable {

		/**
		 * @param issuetypes
		 *            An optional `Issuetypes` array
		 * @return A mapping of randomly selected `IPressure` values to the
		 *         given or all known `Issuetypes`
		 */
		public static Map<Issuetypes, ISatisfaction> requestSatisfactionForIssuesTable(
				Issuetypes... issuetypes) {
			return (Map<Issuetypes, ISatisfaction>) requestTForXTable(
					issuetypes, Clazzes.ISatisfaction);
		}

		/**
		 * @param satisfaction
		 *            An `ISatisfaction` value
		 * @param issuetypes
		 *            An optional array of `Issuetypes`
		 * @return A mapping of the given `ISatisfaction` to the given or all
		 *         known `Issuetypes` values
		 */
		public static Map<Issuetypes, ISatisfaction> requestSatisfactionForIssuesTable(
				ISatisfaction satisfaction, Issuetypes... issuetypes) {
			return requestTForXTable(issuetypes, satisfaction);
		}

		/**
		 * @param satisfaction
		 *            A `Float` value
		 * @param issuetypes
		 *            An optional array of `Issuetypes`
		 * @return A mapping of the given `ISatisfaction` (from `Float`) to the
		 *         given or all known `Issuetypes` values
		 */
		public static Map<Issuetypes, ISatisfaction> requestSatisfactionForIssuetypesTable(
				Float satisfaction, Issuetypes... issuetypes) {
			return requestTForXTable(issuetypes,
					(ISatisfaction) new Satisfaction(satisfaction));
		}
	}

	public static class APressureForIssuesTable {

		/**
		 * @param issuetypes
		 *            An optional `Issuetypes` array
		 * @return A mapping of randomly selected `IPressure` values to the
		 *         given or all known `Issuetypes`
		 */
		public static Map<Issuetypes, IPressure> requestPressureForIssuesTable(
				Issuetypes... issuetypes) {
			return (Map<Issuetypes, IPressure>) requestTForXTable(issuetypes,
					Clazzes.IPressure);
		}

		/**
		 * @param pressure
		 *            An `IPressure` value
		 * @param issuetypes
		 *            An optional array of `Issuetypes`
		 * @return A mapping of the given `IPressure` to the given or all known
		 *         `Issuetypes` values
		 */
		public static Map<Issuetypes, IPressure> requestPressureForIssuesTable(
				IPressure pressure, Issuetypes... issuetypes) {
			return requestTForXTable(issuetypes, pressure);
		}

		/**
		 * @param pressure
		 *            A `Float` value
		 * @param issuetypes
		 *            An optional array of `Issuetypes`
		 * @return A mapping of the given `IPressure` (from `Float`) to the
		 *         given or all known `Issuetypes` values
		 */
		public static Map<Issuetypes, IPressure> requestPressureForIssuetypesTable(
				Float pressure, Issuetypes... issuetypes) {
			return requestTForXTable(issuetypes, (IPressure) new Pressure(
					pressure));
		}
	}

	public static class APressureForProposalTable {

		/**
		 * @param dimensions
		 * @tag usage.parameter : Give `null` here to have configuration driven
		 *      dimensions
		 * @param proposals
		 * @tag usage.parameter : Give an empty array (`new Map<GridPoint,
		 *      CLCover>[0]`) here to have random proposals/arrays
		 * @return A mapping of randomly selected `IPressure` values to the
		 *         given or randomly generated `Map<GridPoint, CLCover>`
		 *         (proposals) values
		 */
		public static Map<Map<GridPoint, CLCover>, IPressure> requestPressureForProposalTable(
				List<Map<GridPoint, CLCover>> proposals) {
			return (Map<Map<GridPoint, CLCover>, IPressure>) (null != proposals
					&& proposals.size() > 0 //
			? requestTForXTable(proposals, Clazzes.IPressure) // else
					: requestTForXTable(randomize1(), Clazzes.IPressure));
		}

		/**
		 * @param pressure
		 *            An `IPressure` value
		 * @param proposals
		 *            An optional map of proposals
		 * @return A mapping of the given or a random `IPressure` to the given
		 *         or randomly generated `Map<GridPoint, CLCover>` (proposals)
		 *         values
		 */
		public static Map<Map<GridPoint, CLCover>, IPressure> requestPressureForProposalTable(
				IPressure pressure, List<Map<GridPoint, CLCover>> proposals) {
			return requestTForXTable(
					null != proposals && proposals.size() > 0 ? proposals
							: randomize1(), pressure);
		}

		/**
		 * @param pressure
		 *            A `Float` value
		 * @param proposals
		 *            An optional map of proposals
		 * @return A mapping of the given or a random `IPressure` (from `Float`)
		 *         to the given or randomly generated `Map<GridPoint, CLCover>`
		 *         (proposals) values
		 */
		public static Map<Map<GridPoint, CLCover>, Pressure> requestPressureForProposalTable(
				Float pressure, List<Map<GridPoint, CLCover>> proposals) {
			return requestTForXTable(
					null != proposals && proposals.size() > 0 ? proposals
							: randomize1(), new Pressure(pressure));
		}
	}

	public static class AProposalsSet {

		/**
		 * @param dimensions
		 * @tag usage.parameter : Give `null` here for `dimensions` to have
		 *      configuration driven
		 * @return A set of randomly selected proposals (proposals here are maps
		 *         with grid point distributions)
		 */
		public static IProposalsSet requestProposalsSet(int[] dimensions) {
			Set<Map<GridPoint, CLCover>> proposalsSet = new HashSet<Map<GridPoint, CLCover>>();

			// for (Map<GridPoint, CLCover> entry : randomize1()) {
			// for (Map.Entry<GridPoint, CLCover> entry2 : entry.entrySet()) {
			// // log().info(entry2.getKey().getX()+","
			// // +entry2.getKey().getY()+ ": " + entry2.getValue());
			// }
			// }

			for (Map<GridPoint, CLCover> map : randomize1()) {
				assertTrue(proposalsSet.add(map));
			}

			// for (Map<GridPoint, CLCover> entry : proposalsSet) {
			// for (Map.Entry<GridPoint, CLCover> entry2 : entry.entrySet()) {
			// // log().info(
			// // entry2.getKey().getX() + ","
			// // + entry2.getKey().getY() + ": "
			// // + entry2.getValue());
			// }
			// }

			return new ProposalsSet(proposalsSet);
		}
	}

	public static class ASatisfactionPerIssueOfAgent { // OK
		static Map<Issuetypes, ISatisfaction> issuemap = new HashMap<Issuetypes, ISatisfaction>();

		private static <E extends Enum<E>> SatisfactionPerIssueOfAgentAllCells<E> delegate(
				Class<E> _enum) {

			for (Issuetypes issue : Issuetypes.values()) {
				issuemap.put(issue, new Satisfaction(Math.random()));
			}

			Collection<Map<E, Map<Issuetypes, ISatisfaction>>> issues = new ArrayList<Map<E, Map<Issuetypes, ISatisfaction>>>();
			int cells = 1;
			for (int dim : INSTANCE.areaDims) {
				cells = cells * dim;
			}
			for (int i = 0; i < cells; i++) {
				Map<E, Map<Issuetypes, ISatisfaction>> _issues = new HashMap<E, Map<Issuetypes, ISatisfaction>>();

				Iterator<E> values = EnumSet.allOf(_enum).iterator();
				while (values.hasNext()) {
					_issues.put(values.next(), issuemap);
				}
				assertTrue(issues.add(_issues));
			}
			return requestMapToGridPoint(
					issues,
					new SatisfactionPerIssueOfAgentAllCells<E>(
							new HashMap<GridPoint, Map<E, Map<Issuetypes, ISatisfaction>>>()),
					INSTANCE.areaDims);
		}

		/**
		 * @return A map filled with randomly generated mappings of
		 *         `ISatisfaction` values
		 */
		public static ISatisfactionPerIssueOfAgentAllCells<Householdtypes> requestSatisfactionPerIssueOfResidentAllCells() {
			return new SatisfactionPerIssueOfAgentAllCells<Householdtypes>(
					delegate(Householdtypes.class));
		}

		/**
		 * @return A map filled with randomly generated mappings of
		 *         `ISatisfaction` values
		 */
		public static ISatisfactionPerIssueOfAgentAllCells<Businesstypes> requestSatisfactionPerIssueOfBusinessAllCells() {
			return new SatisfactionPerIssueOfAgentAllCells<Businesstypes>(
					delegate(Businesstypes.class));
		}
	}

	protected static final GridValueLayer getDefaultGVLayer(String layerName,
			int[] dimensions) {
		return new GridValueLayer(layerName, 3.41, true,
				new repast.simphony.space.grid.StickyBorders(),
				null != dimensions ? dimensions : INSTANCE.areaDims);
	}

	/**
	 * @param <X>
	 *            The type of the key-set elements of the returned map
	 * @param x
	 *            An array of `X` values, number equiv. to `keySet().size()` of
	 *            the returned map
	 * @param satisfaction
	 *            An optional value of type `ISatisfaction` to be mapped at each
	 *            map entry
	 *
	 * @return A map filled with one and the same `x` value mapped to each key
	 *         or randomly generated satisfaction values if no `x` parameter was
	 *         given
	 */
	public static <X> ISatisfactionForXTable<X> requestSatisfactionForXTable(
			X[] x, ISatisfaction... satisfaction) {
		Map<X, ISatisfaction> entries = new HashMap<X, ISatisfaction>();
		if (satisfaction.length < 1) {
			for (X c : x) {
				entries.put(c, new Satisfaction(Math.random()));
			}
		} else {
			entries = requestMap(satisfaction[0], x);
		}
		return new SatisfactionForXTable<X>();
	}

	/**
	 * @param <X>
	 *            The type of the key-set elements of the returned map
	 * @param <T>
	 * @param x
	 *            An array of `X` values, number equiv. to `keySet().size()` of
	 *            the returned map
	 * @param pressure
	 *            An optional value of type `IPressure` to be mapped at each map
	 *            entry
	 * @return A map filled with one and the same `x` value mapped to each key
	 *         or randomly generated pressure values if no `x` parameter was
	 *         given
	 */
	public static <X, T> Map<X, T> requestTForXTable(X[] x, T t) {

		Map<X, T> entries = new HashMap<X, T>();

		entries = requestMap(t, x);

		// CellDumbMock.randomize1(new GridPoint(0, 0), new GridPoint(1, 0),
		// new GridPoint(0, 1));

		return new HashMap<X, T>(entries);
	}

	public static <X, T> Map<X, T> requestTForXTable(List<X> x, T t) {

		Map<X, T> entries = new HashMap<X, T>();

		entries = requestMap(t, x);

		// CellDumbMock.randomize1(new GridPoint(0, 0), new GridPoint(1, 0),
		// new GridPoint(0, 1));

		return new HashMap<X, T>(entries);
	}

	// try : public static <X> ItForXTable<X, ?> requestTForXTable(X[] x,
	// Class<?> t) {...
	public static <X> Map<X, ?> requestTForXTable(X[] x, Clazzes clazz) {

		switch (clazz) {
		case IPressure:
			Map<X, IPressure> m1 = new HashMap<X, IPressure>();
			for (X c : x) {
				m1.put(c, new Pressure(randomizeMinus1t1()));
			}
			return new HashMap<X, IPressure>(m1);
		case ISatisfaction:
			Map<X, ISatisfaction> m2 = new HashMap<X, ISatisfaction>();
			for (X c : x) {
				m2.put(c, new Satisfaction(Math.random()));
			}
			return new HashMap<X, ISatisfaction>(m2);
		case Boolean:
			Map<X, Boolean> m3 = new HashMap<X, Boolean>();
			for (X c : x) {
				m3.put(c, new Boolean(randomize()));
			}
			return new HashMap<X, Boolean>(m3);
		default:
			break;
		}
		return null;
	}

	public static <X> Map<X, ?> requestTForXTable(List<X> x, Clazzes clazz) {

		switch (clazz) {
		case IPressure:
			Map<X, IPressure> m1 = new HashMap<X, IPressure>();
			for (X c : x) {
				m1.put(c, new Pressure(randomizeMinus1t1()));
			}
			return new HashMap<X, IPressure>(m1);
		case ISatisfaction:
			Map<X, ISatisfaction> m2 = new HashMap<X, ISatisfaction>();
			for (X c : x) {
				m2.put(c, new Satisfaction(Math.random()));
			}
			return new HashMap<X, ISatisfaction>(m2);
		case Boolean:
			Map<X, Boolean> m3 = new HashMap<X, Boolean>();
			for (X c : x) {
				m3.put(c, new Boolean(randomize()));
			}
			return new HashMap<X, Boolean>(m3);
		default:
			break;
		}
		return null;
	}

	protected static <K, V> Map<K, V> requestMap(V value, K... keys) {

		Map<K, V> map = new HashMap<K, V>();

		for (K key : keys) {
			map.put(key, value);
		}
		// {{{ XXX DEBUG
		for (Map.Entry<K, V> entry : map.entrySet()) {
			log.trace(entry.getKey() + ", " + entry.getValue());
		}
		// }}}
		return map;
	}

	protected static <K, V> Map<K, V> requestMap(V value, List<K> keys) {

		Map<K, V> map = new HashMap<K, V>();

		for (K key : keys) {
			map.put(key, value);
		}
		// {{{ XXX DEBUG
		for (Map.Entry<K, V> entry : map.entrySet()) {
			log.trace(entry.getKey() + ", " + entry.getValue());
		}
		// }}}
		return map;
	}

	protected static <K, D extends INumber<Float>> Map<K, Double> requestMapDouble(
			K[] keys, D... value) {

		Map<K, Double> map = new HashMap<K, Double>();
		Set<Entry<K, Double>> d = map.entrySet();

		Map.Entry<K, Double> e;

		if (value.length > 0) {
			for (K key : keys) {
				map.put(key, Double.valueOf(value[0].doubleValue()));
			}
		} else {
			for (K key : keys) {
				map.put(key, Double.valueOf(Math.random()));
			}
		}
		// {{{ XXX DEBUG
		// for (Map.Entry<K, V> entry : map.entrySet()) {
		// log().info(entry.getKey() + ", " + entry.getValue());
		// }
		// }}}
		return map;
	}

	protected static <V> Map<GridPoint, V> requestMapToGridPoint(V value) {

		Map<GridPoint, V> map = new HashMap<GridPoint, V>();

		int[] dimX = new int[] { 3, 3 };

		CellAccessor<V, Map<GridPoint, Object>> accessor = new SingleOccupancyCellAccessor<V>();

		for (int dimX_y = 0; dimX_y < dimX[1]; dimX_y++) {
			for (int dimX_x = 0; dimX_x < dimX[0]; dimX_x++) {
				// locationStorage.put(new GridPoint(dimX_x, dimX_y),
				// clCovers.length > 0 ? clCovers[0] : new Integer(2));
				accessor.put(value, (Map<GridPoint, Object>) map,
						new GridPoint(dimX_x, dimX_y));
			}
		}
		// {{{ XXX DEBUG
		// for (Map.Entry<GridPoint, V> entry : map.entrySet()) {
		// log().info(entry.getKey() + ", " + entry.getValue());
		// }
		// }}}
		return map;
	}

	/*
	 * protected static <T> Grid<T> requestFullGridOld(T value, int... dims) {
	 *
	 * int dim1 = (dims.length >= 1) ? dims[0] : 0; int dim2 = (dims.length >=
	 * 2) ? dims[1] : 0;
	 *
	 * GridAdder<T> adder = new RandomGridAdder<T>();
	 *
	 * CellAccessor<T, Map<GridPoint, Object>> accessor = new
	 * SingleOccupancyCellAccessor<T>();
	 *
	 * GridBuilderParameters<T> params = GridBuilderParameters
	 * .singleOccupancy2D(adder, new StrictBorders(), dim1, dim2);
	 *
	 * // locationStorage = fillLocationStorage(accessor, value, params //
	 * .getDimensions());
	 *
	 * Grid<T> grid = new DefaultGrid<T>("Grid", params.getAdder(), params
	 * .getGridPointTranslator(), accessor, params.getDimensions());
	 *
	 * grid = fillGrid(grid, params.getAdder(), value, params.getDimensions());
	 *
	 * // {{{ XXX DEBUG // log().info( // grid.getCellAccessor() //
	 * .get(locationStorage, new GridPoint(1, 2)).toString()); // }}} return
	 * grid; }
	 */

	/*
	 * protected static <T> Grid<T[]> requestFullGridOld(int[] dims, T...
	 * values) {
	 *
	 * int dim1 = (dims.length >= 1) ? dims[0] : 0; int dim2 = (dims.length >=
	 * 2) ? dims[1] : 0;
	 *
	 * GridAdder<T[]> adder = new RandomGridAdder<T[]>();
	 *
	 * // CellAccessor<T[], Map<GridPoint, Object>> accessor = new //
	 * SingleOccupancyCellAccessor<T[]>(); CellAccessor<T[], Map<GridPoint,
	 * Object>> accessor = new MultiOccupancyCellAccessor<T[]>();
	 *
	 * // GridBuilderParameters<T[]> params = GridBuilderParameters //
	 * .singleOccupancy2D(adder, new StrictBorders(), dim1, dim2);
	 * GridBuilderParameters<T[]> params = GridBuilderParameters
	 * .multiOccupancy2D(adder, new StrictBorders(), dim1, dim2);
	 *
	 * // locationStorage = fillLocationStorage(accessor, values, params //
	 * .getDimensions());
	 *
	 * Grid<T[]> grid = new DefaultGrid<T[]>("Grid", params.getAdder(), params
	 * .getGridPointTranslator(), accessor, params.getDimensions());
	 *
	 * grid = fillGrid(grid, params.getAdder(), values, params.getDimensions());
	 *
	 * // {{{ XXX DEBUG // log().info( // grid.getCellAccessor() //
	 * .get(locationStorage, new GridPoint(1, 2)).toString()); // }}} return
	 * grid; }
	 */

	// protected static GridValueLayer requestFullLayer(Collection<Double>
	// values,
	// String vContextName, GridValueLayer vLayer) {
	//
	// // Context<T> context = new DefaultContext<T>(vContextName);
	// // gridSetHierarchy(context);
	// // context.addValueLayer(valueLayer);
	//
	// Iterator<Double> vIterator = values.iterator();
	// // Add the value to all grid positions
	// int xDim = (int) vLayer.getDimensions().getWidth();
	// int yDim = (int) vLayer.getDimensions().getHeight();
	// for (int y = 0; y < yDim; y++) {
	// for (int x = 0; x < xDim; x++) {
	// double value = vIterator.hasNext() ? vIterator.next() : null;
	// vLayer.set(value, x, y);
	// }
	// }
	//
	// return vLayer;
	// }

	/**
	 * @param values
	 *            A Collection<V> of values of type V to be stored in the map
	 * @param map
	 *            A map derivative storing values of type V mapped to
	 *            GridPoint-typed keys
	 * @return M A map filled
	 */
	protected static <M extends Map<GridPoint, V>, V> M requestMapToGridPoint(
			Collection<V> values, M map, int[] dimensions) {

		int[] dims = null != dimensions ? dimensions : INSTANCE.areaDims;

		Iterator<V> vIterator = values.iterator();
		Object _ = null;
		for (int dimX_y = 0; dimX_y < dims[1]; dimX_y++) {
			for (int dimX_x = 0; dimX_x < dims[0]; dimX_x++) {
				V value = vIterator.hasNext() ? vIterator.next() : null;
				// if value is an array
				int[] test = null;
				Arrays.asList(test);

				map.put(new GridPoint(dimX_x, dimX_y), value);
			}
		}
		// {{{ XXX DEBUG
		// for (Map.Entry<GridPoint, V> entry : map.entrySet()) {
		// log().info(entry.getKey() + ", " + entry.getValue());
		// }
		// }}}
		return map;
	}

	protected static <M extends Map<GridPoint, V>, V> M requestMapFromCollection(
			Collection<V> values, M map, GridPoint... points) {

		Iterator<V> vIterator = values.iterator();
		for (GridPoint g : points) {

			V value = vIterator.hasNext() ? vIterator.next() : null;

			map.put(g, value);

		}
		// {{{ XXX DEBUG
		// for (Map.Entry<GridPoint, V> entry : map.entrySet()) {
		// log().info(entry.getKey() + ", " + entry.getValue());
		// }
		// }}}
		return map;
	}

	/**
	 * @param values
	 *            A Collection<V[]> of values of type array of V to be stored in
	 *            the map, so the values at one collection entry will be stored
	 *            at one key in the multimap
	 * @param map
	 *            A Multimap derivative storing multiple values of type V mapped
	 *            to GridPoint-typed keys
	 * @return M the filled Multimap
	 */
	protected static <M extends Multimap<GridPoint, V>, V> M requestMultimapToGridPoint(
			Collection<V[]> values, M map, int[] dimension) {

		Iterator<V[]> vIterator = values.iterator();

		for (int dimX_y = 0; dimX_y < dimension[1]; dimX_y++) {
			for (int dimX_x = 0; dimX_x < dimension[0]; dimX_x++) {
				V[] value = vIterator.hasNext() ? vIterator.next() : null;
				map.putAll(new GridPoint(dimX_x, dimX_y), Arrays.asList(value));
			}
		}
		// {{{ XXX DEBUG
		// for (Map.Entry<GridPoint, V> entry : map.entries()) {
		// log.trace(entry.getKey() + ", " + entry.getValue());
		// }
		// }}}
		return map;
	}

	/**
	 * @FIXME{p:Paper Raster stores NODATA values internally as NaN Double
	 *                instances, we have -9999 as the NODATA representation so
	 *                is it needed to convert them??? AllCells covering whole cs
	 *                area could also store NaN values.}
	 *
	 * @FIXME{FIXED p:Paper We assume that using this method the NaN values
	 *              stored in a Raster are correct/valid in our sense -
	 *              inter-model computations - and therefore are handled,
	 *              internally stored as peripherical points, in terms of our
	 *              API this means calling e. g. new LandPrice(doubleValue,
	 *              true) the boolean value true marking this value comes from a
	 *              peripherific point}
	 *
	 * @param <S>
	 * @param <T>
	 * @param values
	 * @param vLayer
	 * @return
	 */
	protected static <S extends AbstractAllCells<T>, T> S requestFullLayer(
			Raster values, S vLayer) {
		int rowNum = 0;
		for (double[] row : values.getData()) {
			int colNum = 0;
			for (double value : row) {
				vLayer.set(/** value == Double.NaN ? NDATA : */
				value, rowNum, colNum);
				colNum++;
			}
			rowNum++;
		}

		return vLayer;
	}

	/**
	 * Very important note: Here will b assumed that NODATA entries in the
	 * raster lead to accepted entries as they are referred to by concept as
	 * peripherific
	 *
	 * @param <S>
	 * @param <T>
	 * @param values
	 * @param vLayer
	 * @return
	 * @throws ClassNotFoundException
	 */
	@SuppressWarnings("unchecked")
	private static <S extends AbstractAllCells<I>, I, T extends I> S requestFullLayer(
			Raster values, S vLayer, T theNODATA) {

		// find out per refelction which implementation class T has
		Method m = null;
		int rowNum = 0;
		for (double[] row : values.getData()) {
			int colNum = 0;
			for (double value : row) {

				try {
					m = theNODATA.getClass().getMethod("getInstance",
							Number.class,
							Array.newInstance(boolean.class, 0).getClass());
					vLayer
							.set(0 == Double.valueOf(value).compareTo(
									Double.NaN) ? (I) theNODATA : (I) m.invoke(
									theNODATA, value, new boolean[] { false }),
									rowNum, colNum);
				} catch (Throwable t) {
					handleException(CellDumbMock.class, t);
					System.exit(0);
				}
				colNum++;
			}
			rowNum++;
		}

		return vLayer;
	}

	protected static <S extends AbstractAllCells<T>, T> S requestFullLayer(
			Map<GridPoint, T> values, S vLayer) {

		// Add the value to all grid positions
		for (Entry<GridPoint, T> entry : values.entrySet()) {
			vLayer.set(entry.getValue(), entry.getKey().toIntArray(null));
		}

		return vLayer;
	}

	protected static <S extends AbstractAllCells<T>, T> S requestFullLayer(
			Collection<T> values, S vLayer) {

		Iterator<T> vIterator = values.iterator();
		// Add the value to all grid positions
		int xDim = (int) vLayer.getDimensions().getWidth();
		int yDim = (int) vLayer.getDimensions().getHeight();
		for (int y = 0; y < yDim; y++) {
			for (int x = 0; x < xDim; x++) {
				T value = vIterator.hasNext() ? vIterator.next() : null;
				vLayer.set(value, x, y);
			}
		}

		return vLayer;
	}

	// protected static <T> Grid<T> requestFullGrid(Collection<T> values,
	// String vContextName, String vGridName, int... dims) {
	// Context<T> context = new DefaultContext<T>(vContextName);
	// gridSetHierarchy(context);
	// gridAssociateProjection(context, vGridName, false);
	// Grid<T> grid = (Grid<T>) context.getProjection(vGridName);
	// Iterator<T> vIterator = values.iterator();
	// // Add the value to all grid positions
	// int xDim = grid.getDimensions().getWidth();
	// int yDim = grid.getDimensions().getHeight();
	// for (int y = 0; y < yDim; y++) {
	// for (int x = 0; x < xDim; x++) {
	// T value = vIterator.hasNext() ? vIterator.next() : null;
	// gridAddObject(context, value);
	// assertTrue(grid.moveTo(value, x, y));
	// assertEquals(x, grid.getLocation(value).getX());
	// assertEquals(y, grid.getLocation(value).getY());
	// }
	// }
	//
	// assertEquals(values.size(), grid.size());
	//
	// return grid;
	// }

	// protected static <T> Grid<T[]> requestFullGrid(int[] dims,
	// String vContextName, String vGridName, T... values) {
	// Context<T[]> context = new DefaultContext<T[]>(vContextName);
	// gridSetHierarchy(context);
	// gridAssociateProjection(context, vGridName, true);
	// Grid<T[]> grid = (Grid<T[]>) context.getProjection(vGridName);
	// gridAddObject(context, values);
	//
	// // FIXME the loop over the cells; comparing cell count with grid.size()
	//
	// GridPoint oLoc = grid.getLocation(values);
	// int xNew = Math.abs(Math.round(oLoc.getX() / 2));
	// int yNew = Math.abs(Math.round(oLoc.getY() / 2));
	// grid.moveTo(values, xNew, yNew);
	// assertEquals(new GridPoint(xNew, yNew), grid.getLocation(values));
	//
	// return grid;
	// }

	// /**
	// * Call like
	// *
	// * <pre>
	// * requestFullSi2DGridContext(&quot;/main/sub1&quot;, &quot;test
	// grid&quot;, 3, 3);
	// * </pre>
	// *
	// * The context path string looks like "/main/sub1/sub2" or
	// * "/main/sub1/sub2", where "main" is root context and "sub2" is the
	// grid's
	// * context to be added to "main/sub1", which has to exist.
	// *
	// * @param <T>
	// * @param contextpath
	// * @param grid
	// * @param dims
	// * @return A context of specified size
	// */
	// public static <T> Context<T> requestFullSi2DGridContext(String
	// contextpath,
	// String grid, int... dims) {
	//
	// String[] contexts = (contextpath.startsWith("/")) ? contextpath
	// .substring(1).split("/", -987) : contextpath.split("/", -987);
	//
	// Context<T> context = new DefaultContext<T>(
	// contexts[contexts.length - 1]);
	//
	// String parent = "";
	//
	// for (int i = 0; i < contexts.length - 1; i++) {
	// parent = parent.concat("/" + contexts[i]);
	// // log().info(parent + " for: " + contexts[contexts.length - 1]);
	// }
	//
	// if (FindContext(contexts[0]) == null) {
	// gridSetUp0(contexts[0]);
	// }
	// gridSetHierarchy0(context, parent + "/");
	// gridAssociateProjection(context, grid, false);
	// return context;
	// }

	protected static <T> ValueLayer requestFullValueLayer(Collection<T> values,
			String vGridName, int... dims) {

		Iterator<T> vIterator = values.iterator();

		GridValueLayer grid = new GridValueLayer(vGridName, true, dims);

		// Add the value to all grid positions
		for (int x = 0; x < grid.getDimensions().getWidth(); x++) {
			for (int y = 0; y < grid.getDimensions().getHeight(); y++) {
				Double value = (Double) (vIterator.hasNext() ? vIterator.next()
						: 0);
				grid.set(value, x, y);
				// Assert.assertEquals(value, grid.get(x, y));
			}
		}

		for (int x = 0; x < grid.getDimensions().getWidth(); x++) {
			for (int y = 0; y < grid.getDimensions().getHeight(); y++) {
				log.debug(grid.get(x, y) + " at [" + x + ", " + y + "]");
			}
		}

		/**
		 * Is a `ValueLayer` only, has no `set()` method, XXX is it better to
		 * return a full-featured GridValueLayer here?
		 */
		return grid;
	}

	@VisibleForTesting
	static AbstractAllCells<Boolean> testFillLayerDirectExample() {

		return requestFullLayer(fill(INSTANCE.areaDims, true),
				new BooleanAllCells(new GridValueLayer("testing", 1, true,
						new repast.simphony.space.grid.StickyBorders(),
						INSTANCE.areaDims)));
	}

	@VisibleForTesting
	static <T> void printValueLayer(AbstractAllCells<T> call) {
		int xDim = (int) call.getDimensions().getWidth();
		int yDim = (int) call.getDimensions().getHeight();
		for (int y = 0; y < yDim; y++) {
			for (int x = 0; x < xDim; x++) {
				log.debug("loc: " + x + ", " + y + " <- " + call.get(x, y));
			}
		}
	}

	@VisibleForTesting
	static <T> void printGridPointMap(Map<GridPoint, T> map) {
		for (Map.Entry<GridPoint, T> entry : map.entrySet()) {
			log.debug("loc: " + entry.getKey().getX() + ", "
					+ entry.getKey().getY() + " <- " + entry.getValue());
		}
	}

	@VisibleForTesting
	static <T> void printGridPointMultimap(Multimap<GridPoint, T> map) {

		StringBuffer sb = new StringBuffer('\n');

		for (GridPoint key : map.keySet()) {
			sb.append("loc: " + key.getX() + ", " + key.getY() + " <- ");
			for (T value : map.get(key)) {
				sb.append(value + ", ");
			}
			sb.deleteCharAt(sb.length() - 1);
			sb.setCharAt(sb.length() - 1, '.');
			sb.append('\n');
		}
		log.debug(sb);
	}

	// @VisibleForTesting
	// static <K, T> void printMap(Map<K, T> map) {
	//
	// for (Map.Entry<K, T> entry : map.entrySet()) {
	// log.debug("key: " + entry.getKey() + " <- " + entry.getValue());
	// }
	// }

	@VisibleForTesting
	static <K, T, M extends Map<K, T>> T printMap(M map) {

		for (Map.Entry<K, T> entry : map.entrySet()) {

			T v = entry.getValue();
			if (v.getClass().getSuperclass() == AbstractBoundedFloat.class) {
				log.debug("key: " + entry.getKey() + " <- "
						+ ((AbstractBoundedFloat) v).value());
			} else if (v.getClass().getSuperclass() == AbstractBoundedInteger.class) {
				log.debug("key: " + entry.getKey() + " <- "
						+ ((AbstractBoundedInteger) v).value());
			} else {
				log.debug("key: " + entry.getKey() + " <- " + v);
			}

		}
		return null;
	}

	@VisibleForTesting
	static <K, T extends Enum<T>, M extends Map<K, T>> T printMap(M map) {

		for (Map.Entry<K, T> entry : map.entrySet()) {
			log.debug("key: " + entry.getKey() + " <- " + entry.getValue());
		}
		return null;
	}

	@VisibleForTesting
	static <E, S extends Map<?, ?>> void printSet(Set<E> set) {
		for (E entry : set) {
			log.debug("entry: " + entry.toString());
		}
	}

	/**
	 * @return An array of 5 randomly selected proposals (of type
	 *         `Map<GridPoint, CLCover>`) at each grid point
	 */
	public static List<Map<GridPoint, CLCover>> randomize1(GridPoint... points) {
		int cap = 5;
		GridPoint[] pts = null != points && points.length > 0 ? points
				: new GridPoint[] { new GridPoint(0, 0), new GridPoint(0, 1),
						new GridPoint(1, 0) };
		// created list has no entries but capacity of 5
		List<Map<GridPoint, CLCover>> clc = new ArrayList<Map<GridPoint, CLCover>>(
				cap);

		for (int i = 0; i < cap; i++) {
			assertTrue(clc.add(ACLCoverSomeCells.requestCLCoverSomeCells(pts)));
		}
		return clc;
	}
}
