/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import java.awt.Event;
import java.util.EventObject;

public class DataEvent extends EventObject {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The event's id.
	 * 
	 * @serial
	 * @see #getID()
	 * @see #DataEvent
	 */
	private int id;

	Object variable;

	public static final int DATA_CHANGED = 8000;

	/**
	 * Constructs an DataEvent object from the parameters of a 1.0-style event.
	 * 
	 * @param event
	 *            the old-style event
	 */
	public DataEvent(Event event) {
		this(event.target, event.id, new Object() {
			@Override
			public String toString() {
				// TODO Auto-generated method stub
				return "No variable for that "
						+ this.getClass().getSimpleName() + " "
						+ super.toString();
			}
		});
	}

	/**
	 * Constructs an DataEvent object with the specified source object and type.
	 * 
	 * @param source
	 *            the object where the event originated
	 * @param id
	 *            the event type
	 * @param variable
	 *            the variable object holding the data changed
	 */
	public DataEvent(Object source, int id, Object variable) {
		super(source);
		this.id = id;
		this.variable = variable;
		switch (id) {
		// case ActionEvent.ACTION_PERFORMED:
		// case ItemEvent.ITEM_STATE_CHANGED:
		// case AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED:
		// case TextEvent.TEXT_VALUE_CHANGED:
		// consumed = true;
		// break;
		default:
		}
	}

	/**
	 * Returns the event type.
	 */
	public int getID() {
		return id;
	}

	public Object getVariable() {
		return variable;
	}
}
