/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import static java.util.logging.Logger.getLogger;

import java.util.Enumeration;
import java.util.Vector;
import java.util.logging.Logger;

import abmland.common.DataEvent;
import abmland.common.DataListener;

public class DataEventMgr {

	private final static Logger log = getLogger(DataEventMgr.class.getName());

	private static Vector<DataListener> listeners = new Vector<DataListener>();

	public static synchronized void addDataChangeListener(Object eventSource,
			DataListener d) {
		listeners.addElement(d);
		Object added = listeners.get(listeners.lastIndexOf(d));
		log.info(eventSource.getClass().getSimpleName() + ": "
				+ added.getClass().getSimpleName() + " (" + added.hashCode()
				+ ") registered.");
	}

	public static synchronized void removeDataChangeListener(DataListener d) {
		listeners.removeElement(d);
	}

	public static void notifyListeners(DataEvent evt) {
		DataListener datalistener;
		for (Enumeration<DataListener> e = listeners.elements(); e
				.hasMoreElements();) {
			datalistener = e.nextElement();
			switch (evt.getID()) {
			case DataEvent.DATA_CHANGED:
				datalistener.dataChanged(evt);
				break;
			default:
				break;
			}
		}
		return;
	}
}
