/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class DefaultLayer<T> implements Layer<T> {

	protected List<LayerListener<T>> listeners = new ArrayList<LayerListener<T>>();

	/** {@inheritedDoc} */
	public void addLayerListener(LayerListener<T> listener) {
		listeners.add(listener);
	}

	/** {@inheritedDoc} */
	public Collection<LayerListener<T>> getLayerListeners() {
		return listeners;
	}

	/** {@inheritedDoc} */
	public boolean removeLayerListener(LayerListener<T> listener) {
		return listeners.remove(listener);
	}

	/**
	 * This method is called from this / - potentially the deriving
	 * `Layer`-typed class - to broadcast the raised `LayerEvent` to all
	 * `LayerListener`-implementors. If the deriving class may not access this method,
	 * it may call `layerEventOccurred(evt);` itself, like:
	 *
	 * 		Collection&lt;LayerListener&lt;T&gt;&gt; listeners = getLayerListeners();
	 * 		for (LayerListener&lt;T&gt; layerListener : listeners) {
	 * 			layerListener.layerEventOccurred(evt);
	 * 		}
	 *
	 * @param evt
	 */
	protected void fireLayerEvent(LayerEvent<T> evt) {
		// OPTIMIZED: code this out for speed because this gets called a lot
		// (potentially)
		for (LayerListener<T> layerListener : listeners) {
			layerListener.layerEventOccurred(evt);
		}
	}
}
