/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import static org.apache.log4j.Logger.getLogger;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;
import org.junit.Test;

import repast.simphony.parameter.StringConverter;

/**
 * If implemented, please put into non-test class path...
 *
 * @author dkahlenb
 */
public class DoubleArrayConverter implements StringConverter/** <Double[]> */
{

	private final Logger log = getLogger(getClass());

	/**
	 * XXX we needed double[], so Double[] wasn't suitable, because Integer[]
	 * not cast-able to double[] easily TODO How to re-get type safety
	 * features...
	 */
	// /** {@inheritDoc} */
	// public String toString(Double[] obj) {
	// StringBuffer sb = new StringBuffer();
	// for (Double a : obj) {
	// sb.append(a + " ");
	// }
	// return sb.deleteCharAt(sb.length() - 1).toString();
	// }

	/**
	 * XXX we needed double[], so Double[] wasn't suitable, because Integer[]
	 * not cast-able to double[] easily TODO How to re-get type safety
	 * features...
	 */
	// /** {@inheritDoc} */
	// public Double[] fromString(String strRep) {
	// List<Double> list = new ArrayList<Double>();
	// Pattern enumTypeRegexPattern = Pattern
	// .compile("([+-]?\\d*\\.?[Ee]?[+-]?\\d*) *?");
	// Matcher enumTypeRegexMatcher = enumTypeRegexPattern.matcher(strRep);
	// int j = 0;
	// while (enumTypeRegexMatcher.find()) {
	// j++;
	// for (int i = 0; i < enumTypeRegexMatcher.groupCount(); i++) {
	// if (j % 2 != 0) {
	// // log.debug("group " + i + ": " +
	// // enumTypeRegexMatcher.group(i));
	// list.add(Double.parseDouble(enumTypeRegexMatcher.group(i)));
	// }
	// }
	// }
	// return list.toArray(new Double[0]);
	// }

	/** {@inheritDoc} */
	public String toString(Object obj) {
		StringBuffer sb = new StringBuffer();
		for (Double a : (Double[]) obj) {
			sb.append(a + " ");
		}
		return sb.deleteCharAt(sb.length() - 1).toString();
	}

	/** {@inheritDoc} */
	public double[] fromString(String strRep) {
		List<Double> list = new ArrayList<Double>();
		Pattern enumTypeRegexPattern = Pattern
				.compile("([+-]?\\d*\\.?[Ee]?[+-]?\\d*) *?");
		Matcher enumTypeRegexMatcher = enumTypeRegexPattern.matcher(strRep);
		int j = 0;
		while (enumTypeRegexMatcher.find()) {
			j++;
			for (int i = 0; i < enumTypeRegexMatcher.groupCount(); i++) {
				if (j % 2 != 0) {
					// log.debug("group " + i + ": " +
					// enumTypeRegexMatcher.group(i));
					list.add(Double.parseDouble(enumTypeRegexMatcher.group(i)));
				}
			}
		}
		j = 0;
		double[] doubleArray = new double[list.size()];
		for (Double d : list) {
			doubleArray[j++] = d;
		}
		return doubleArray;
	}

	@Test
	public void runFromString() {
		try {
			double dumbsum = 0;
			for (Double double1 : fromString("1.0 -2.0 3.0")) {
				dumbsum += double1;
				log.debug(double1);
			}
			log.debug("Sum of floats: " + dumbsum);
		} catch (Exception e) {
			log.error(e.getStackTrace()[0]);
		}
	}
}
