/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import java.io.File;
import java.io.IOException;

import static org.junit.Assert.*;

import org.apache.log4j.Logger;

import repast.simphony.batch.BatchScenarioLoader;
import repast.simphony.engine.controller.Controller;
import repast.simphony.engine.controller.DefaultController;
import repast.simphony.engine.controller.DefaultControllerRegistry;
import repast.simphony.engine.environment.AbstractRunner;
import repast.simphony.engine.environment.ControllerAction;
import repast.simphony.engine.environment.ControllerRegistry;
import repast.simphony.engine.environment.DefaultRunEnvironmentBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunEnvironmentBuilder;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.Schedule;
import repast.simphony.parameter.ParameterConstants;
import repast.simphony.parameter.Parameters;
import repast.simphony.parameter.ParametersCreator;
import repast.simphony.ui.GUIParametersManager;

/**
 * Implementation of `FileInitializableScheduleRunner`
 *
 * One usage shown in `ExternalParametersTesting` (src/test/java)
 *
 * To get the parameter setting correct too, adapted
 * `repast.simphony.ui.RSApplication.initSim()`, which calls `repast
 * .simphony.engine.controller.Controller.runParameterSetters(Parameters)`
 *
 * @author dkahlenb
 *
 */
public class DummyScheduleRunner extends AbstractRunner implements
		FileInitializableScheduleRunner {

	private final Logger log = Logger.getLogger(DummyScheduleRunner.class);

	private RunEnvironmentBuilder runEnvironmentBuilder;
	protected Controller controller;
	private ISchedule schedule;
	private GUIParametersManager paramsManager;

	public DummyScheduleRunner() {
		runEnvironmentBuilder = new DefaultRunEnvironmentBuilder(this, true);
		assertNotNull(runEnvironmentBuilder);
		controller = new DefaultController(runEnvironmentBuilder);
		assertNotNull(controller);

		controller.setScheduleRunner(this);
		// controller.setControllerRegistry(new DefaultControllerRegistry());

		// controller.batchInitialize();
		// controller.runInitialize(null);
	}

	@Deprecated
	public DummyScheduleRunner(String arg) {
		if (!arg.equalsIgnoreCase("Yes I want to fix this constructor.")) {
			new DummyScheduleRunner();
			return;
		}

		runEnvironmentBuilder = new DefaultRunEnvironmentBuilder(this, true);
		controller = new DefaultController(runEnvironmentBuilder);
		controller.setScheduleRunner(this);

		ControllerRegistry registry = new DefaultControllerRegistry();
		registry.setMasterContextId("root");

		// FIXME
		registry.addAction("root", null, new ControllerAction() {

			public void batchCleanup(RunState runState, Object contextId) {
				// TODO Auto-generated method stub

			}

			public void batchInitialize(RunState runState, Object contextId) {

			}

			public void runCleanup(RunState runState, Object contextId) {
				// TODO Auto-generated method stub

			}

			public void runInitialize(RunState runState, Object contextId,
					Parameters runParams) {
				log.info("test");
			}
		});

		assertNotNull(registry.getActionTree("root"));

		controller.setControllerRegistry(registry);
		ParametersCreator creator = new ParametersCreator();
		int val = (int) System.currentTimeMillis();
		if (val < 0)
			val = Math.abs(val);
		creator.addParameter(ParameterConstants.DEFAULT_RANDOM_SEED_USAGE_NAME,
				ParameterConstants.DEFAULT_RANDOM_SEED_DISPLAY_NAME, int.class,
				val, false);
		// parameters.setValue("AssayArea", new Integer[] { 3, 3 });

		// creator.addParameter("AssayArea", Integer[].class,
		// new Integer[] { 3, 3 }, false);
		Parameters params = creator.createParameters();
		// Parameters params = loader.getParameters();
		paramsManager = new GUIParametersManager(params, null);
		registry.addParameterSetter(paramsManager);
		controller.batchInitialize();
		controller.runParameterSetters(paramsManager.getParameters());
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.common.FileInitializedScheduleRunner#load(java.io.File)
	 */
	public void load(File scenarioDir) throws Throwable {
		if (scenarioDir.exists()) {
			BatchScenarioLoader loader = null;
			ControllerRegistry registry = null;
			try {
				loader = new BatchScenarioLoader(scenarioDir);
				registry = loader.load(runEnvironmentBuilder);
			} catch (Throwable t) {
				log.error("Error with loader.", t);
			}
			controller.setControllerRegistry(registry);

			/**
			 * {{{ [Parameter-Loading]
			 */
			Parameters params = loader.getParameters();
			paramsManager = new GUIParametersManager(params, null);
			registry.addParameterSetter(paramsManager);
			/**
			 * }}} [Parameter-Loading]
			 */

		} else {
			// msgCenter.error(...);
			log.error("Scenario folder not found: "
					+ scenarioDir.getAbsolutePath(),
					new IllegalArgumentException("Invalid scenario "
							+ scenarioDir.getAbsolutePath()));
			return;
		}
		// controller.batchInitialize();
		// /**
		// * {{{ [Parameter-Loading]
		// */
		// controller.runParameterSetters(paramsManager.getParameters());
		// /**
		// * }}} [Parameter-Loading]
		// */
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.common.FileInitializedScheduleRunner#batchInitialize()
	 */
	public void batchInitialize() {
		// assertNotNull(controller);
		controller.batchInitialize();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.common.FileInitializedScheduleRunner#paramsInitialize()
	 */
	public void paramsInitialize() {
		if (controller.getNextBatchNumber() < 2) {
			log
					.warn("To get a repast.simphony.parameter.Parameters object, run batchInitialize prior paramsInitialize!");
		}
		/**
		 * {{{ [Parameter-Loading]
		 */
		controller.runParameterSetters(paramsManager.getParameters());
		/**
		 * }}} [Parameter-Loading]
		 */
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.common.FileInitializedScheduleRunner#runInitialize()
	 */
	public void runInitialize() {
		/**
		 * {{{ [Parameter-Loading]
		 */
		// RunState runState = controller.runInitialize(new
		// DefaultParameters());
		RunState runState = controller.runInitialize(paramsManager
				.getParameters());

		// controller.runParameterSetters(null);

		/**
		 * }}} [Parameter-Loading]
		 */
		schedule = runState.getScheduleRegistry().getModelSchedule();
		assertNotNull(schedule);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.common.FileInitializedScheduleRunner#cleanUpRun()
	 */
	public void cleanUpRun() {
		controller.runCleanup();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.common.FileInitializedScheduleRunner#cleanUpBatch()
	 */
	public void cleanUpBatch() {
		controller.batchCleanup();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.common.FileInitializedScheduleRunner#getNextScheduledTime()
	 */
	public double getNextScheduledTime() {
		return ((Schedule) RunEnvironment.getInstance().getCurrentSchedule())
				.peekNextAction().getNextTime();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.common.FileInitializedScheduleRunner#getModelActionCount()
	 */
	public int getModelActionCount() {
		return schedule.getModelActionCount();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.common.FileInitializedScheduleRunner#getActionCount()
	 */
	public int getActionCount() {
		return schedule.getActionCount();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.common.FileInitializedScheduleRunner#step()
	 */
	public void step() {
		schedule.execute();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.common.FileInitializedScheduleRunner#stop()
	 */
	public void stop() {
		if (schedule != null)
			schedule.executeEndActions();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.common.FileInitializedScheduleRunner#setFinishing(boolean)
	 */
	public void setFinishing(boolean fin) {
		schedule.setFinishing(fin);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * abmland.common.FileInitializedScheduleRunner#execute(repast.simphony.
	 * engine.environment.RunState)
	 */
	public void execute(RunState toExecuteOn) {
	}

	private Parameters setupSweep() throws IOException {
		Parameters params = null;
		ControllerRegistry registry = controller.getControllerRegistry();
		// if (producer != null) {
		// producer.init(registry, registry.getMasterContextId());
		// params = producer.getParameters();
		// registry.addParameterSetter(producer.getParameterSweeper());
		// }

		if (params == null) {
			ParametersCreator creator = new ParametersCreator();
			params = creator.createParameters();
		}

		if (!params.getSchema().contains(
				ParameterConstants.DEFAULT_RANDOM_SEED_USAGE_NAME)) {
			ParametersCreator creator = new ParametersCreator();
			creator.addParameters(params);
			creator.addParameter(
					ParameterConstants.DEFAULT_RANDOM_SEED_USAGE_NAME,
					Integer.class, (int) System.currentTimeMillis(), false);
			params = creator.createParameters();
		}

		return params;
	}
}