/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import static org.apache.log4j.Logger.getLogger;
import static org.junit.Assert.*;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;
import org.junit.Test;

import repast.simphony.parameter.StringConverter;
import abmland.data.CLCover;

/**
 * If implemented, please put into non-test class path...
 *
 * @author dkahlenb
 * @param <E>
 */
public class EnumConverterStub<E extends Enum<E>> implements
		StringConverter<Enum<E>> {

	private final Logger log = getLogger(getClass());

	/**
	 * Creates an enum constant object from a String representation.
	 *
	 * @param strRep
	 *            the string representation
	 * @return the created Object.
	 */
	/** {@inheritDoc} */
	public Enum<E> fromString(String strRep) {

		Class<E> enumType = null; // parse from strRep

		// Class.forName(enumType.getCanonicalName());

		Pattern enumTypeRegexPattern = Pattern
				.compile("^(\\S+)\\.{1}(([\\S-[\\.]]+))");

		Matcher enumTypeRegexMatcher = enumTypeRegexPattern.matcher(strRep);

		assertTrue(enumTypeRegexMatcher.find());

//		for (int i = 0; i < enumTypeRegexMatcher.groupCount(); i++) {
//			log.debug("group " + i + ": " + enumTypeRegexMatcher.group(i));
//		}

		try {
			enumType = (Class<E>) Class.forName(enumTypeRegexMatcher.group(1));
		} catch (ClassNotFoundException e) {
			log.error(e.getStackTrace()[0]);
		}

		return Enum.valueOf(enumType, enumTypeRegexMatcher.group(2));
	}

	/** {@inheritDoc} */
	public String toString(Enum obj) {
		return obj.getDeclaringClass().getCanonicalName() + "."
				+ obj.toString();
	}
}
