/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import java.io.File;

import repast.simphony.engine.environment.RunState;

/**
 * A Simphony runner which can i. e. load parameters from extended_params.xml in
 * a .rs folder, evaluate the model.score file and act upon etc. In short it
 * should behave like the Runner started when using the launch files provided by
 * the Simphony distribution.
 */

public interface FileInitializableScheduleRunner {

	public abstract void load(File scenarioDir) throws Throwable;

	public abstract void runInitialize();

	public abstract void batchInitialize();

	public abstract void paramsInitialize();

	public abstract void cleanUpRun();

	public abstract void cleanUpBatch();

	/**
	 * @return returns the tick count of the next scheduled item
	 */
	public abstract double getNextScheduledTime();

	/**
	 * @return returns the number of model actions on the schedule
	 */
	public abstract int getModelActionCount();

	/**
	 * @return returns the number of non-model actions on the schedule
	 */
	public abstract int getActionCount();

	/**
	 * Step the schedule
	 */
	public abstract void step();

	/**
	 * stop the schedule
	 */
	public abstract void stop();

	/**
	 * Set the finishing value for the schedule
	 *
	 * @param fin
	 */
	public abstract void setFinishing(boolean fin);

	public abstract void execute(RunState toExecuteOn);
}