/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import static org.apache.log4j.Logger.getLogger;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

import repast.simphony.parameter.StringConverter;

/**
 * Redundant, look here
 * `repast.simphony.parameter.ParameterUtils.parseDefaultValue(Class, String,
 * String)` and here
 * `repast.simphony.parameter.ListTokenizer.parseIntValues(String, String)`
 *
 * @author dkahlenb
 *
 */
public class IntegerArrayConverter implements StringConverter<List<Integer>> {

	private final Logger log = getLogger(getClass());

	/**
	 * XXX we needed int[], so Integer[] wasn't suitable, because Integer[] not
	 * cast-able to int[] easily
	 */
	// /** {@inheritDoc} */
	// public String toString(Integer[] obj) {
	// StringBuffer sb = new StringBuffer();
	// for (Integer a : obj) {
	// sb.append(a + " ");
	// }
	// return sb.deleteCharAt(sb.length() - 1).toString();
	// }

	/** {@inheritDoc} */
	public String toString(List<Integer> obj) {
		StringBuffer sb = new StringBuffer();
		for (Integer a : obj) {
			sb.append(a + " ");
		}
		return sb.deleteCharAt(sb.length() - 1).toString();
	}

	/**
	 * XXX we needed int[], so Integer[] wasn't suitable, because Integer[] not
	 * cast-able to int[] easily
	 */
	// /** {@inheritDoc} */
	// public Integer[] fromString(String strRep) {
	// List<Integer> list = new ArrayList<Integer>();
	// Pattern enumTypeRegexPattern = Pattern
	// .compile("([+-]?\\d*\\.?[Ee]?[+-]?\\d*) *?");
	// Matcher enumTypeRegexMatcher = enumTypeRegexPattern.matcher(strRep);
	// int j = 0;
	// while (enumTypeRegexMatcher.find()) {
	// j++;
	// for (int i = 0; i < enumTypeRegexMatcher.groupCount(); i++) {
	// if (j % 2 != 0) {
	// // log.debug("group " + i + ": " +
	// // enumTypeRegexMatcher.group(i));
	// list.add(new Double(enumTypeRegexMatcher.group(i)).intValue());
	// }
	// }
	// }
	// return list.toArray(new Integer[0]);
	// }

	public List<Integer> fromString(String strRep) {

		List<Integer> list = new ArrayList<Integer>();
		Pattern enumTypeRegexPattern = Pattern
				.compile("([+-]?\\d*\\.?[Ee]?[+-]?\\d*) *?");
		Matcher enumTypeRegexMatcher = enumTypeRegexPattern.matcher(strRep);
		int j = 0;
		while (enumTypeRegexMatcher.find()) {
			j++;
			for (int i = 0; i < enumTypeRegexMatcher.groupCount(); i++) {
				if (j % 2 != 0) {
					// log.debug("group " + i + ": " +
					// enumTypeRegexMatcher.group(i));
					list.add(new Double(enumTypeRegexMatcher.group(i))
							.intValue());
				}
			}
		}
		return list;
	}
}
