/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import java.util.Collection;

/**
 * Compare to `repast.simphony.space.projection.Projection`. Evt. make a `Layer&lt;T&gt;` out of it.
 *
 * @author dkahlenb
 * @param <T>
 *
 */
public interface Layer<T> {
	/**
	 * Adds listener for this layer.
	 *
	 * @param listener the listener to add.
	 */
	void addLayerListener(LayerListener<T> listener);

	/**
	 * Removes a listener from this layer.
	 *
	 * @param listener the listener to remove
	 * @return true if the listener was succesfully removed, otherwise false
	 */
	boolean removeLayerListener(LayerListener<T> listener);

	/**
	 * Gets all the listeners for this layer.
	 *
	 * @return an iterable over all the listeners for this layer.
	 */
	Collection<LayerListener<T>> getLayerListeners();
}
