/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;


/**
 * Compare to `repast.simphony.space.projection.ProjectionEvent`
 *
 * Eventually make a `LayerEvent&lt;T&gt;` out of it
 *
 * @author dkahlenb
 * @param <T>
 *
 */
public class LayerEvent<T> {
	public enum Type {
		OBJECT_ADDED, OBJECT_REMOVED, OBJECT_MOVED, OBJECT_INVALID
	};

	public static Type OBJECT_ADDED = Type.OBJECT_ADDED;
	public static Type OBJECT_REMOVED = Type.OBJECT_REMOVED;
	public static Type OBJECT_MOVED = Type.OBJECT_MOVED;
	public static Type OBJECT_INVALID = Type.OBJECT_INVALID;

	private Type type;
	private Layer<T> layer;
	/**
	 * Was `private Object subject;`
	 */
	private double subject;

	/**
	 * Imaging a `Layer&lt;T&gt;`-typed class implementing a `LayerListener`. If
	 * you're calling `set(...)` on it, a layer event should be raised, so you
	 * embed a `layerEventOccurred(new LayerEvent(...))` call in this set
	 * method's tail, like:
	 *
	 * ``public void set(double value, int... coordinate) { layerEventOccurred(
	 * new LayerEvent&lt;ISpace&gt;(this, value, Type.OBJECT_ADDED)); }``
	 *
	 * There you see the constructor called with a valid interpretation of it's
	 * parameters.
	 *
	 * Was `public LayerEvent(Layer&lt;T&gt; layer, Object subject, Type type);`
	 *
	 * @param layer
	 *            an object storing which `LayerListener` implementor raised
	 *            this `LayerEvent`
	 *
	 * @param subject
	 *            an object which is of the type, `Layer` types can store -- or
	 *            `null`. More abstract: Which is added, removed or moved. Even
	 *            more abstract: Which is concerned by `LayerEvent.Type` typed events.
	 * @param type
	 *            the type of layer event - one of `Type.OBJECT_ADDED`,
	 *            `Type.OBJECT_REMOVED` or `Type.OBJECT_MOVED` - this `LayerEvent` has
	 */
	public LayerEvent(Layer<T> layer, double subject, Type type) {
		this.layer = layer;
		this.subject = subject;
		this.type = type;
	}

	/**
	 *
	 * @return the event-bus-enabled ... layer storage, i. e. Depends how you
	 *         associate the interface
	 */
	public Layer<T> getLayer() {
		return layer;
	}

	/**
	 *
	 * @return the type of `LayerEvent` raised.
	 */
	public Type getType() {
		return type;
	}

	/**
	 * Was `public Object getSubject();`
	 *
	 * @return the object of type `double` assigned to the i. e.
	 *         `Type.OBJECT_ADDED`, `Type.OBJECT_REMOVED` or `Type.OBJECT_MOVED`
	 *         layer event, in human terms: the object added, removed or moved
	 *         (nearby raising this layer event).
	 */
	public double getSubject() {
		return subject;
	}
}
