/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import java.util.Collection;
import java.util.List;

import abmland.types.AbstractBoundedDouble;
import abmland.types.AbstractBoundedInteger;
import abmland.types.IHouseholdNumber;
import abmland.types.IPressure;
import abmland.types.ISatisfaction;
import abmland.types.ISpace;
import abmland.types.ISupply;
import abmland.types.impl.HouseholdNumber;
import abmland.types.impl.Pressure;
import abmland.types.impl.Satisfaction;
import abmland.types.impl.Space;
import abmland.types.impl.Supply;

public class TypesUtils {

	final static int IANE = 0;
	final static float FANE = 0.0f;
	final static int IMNE = 1;
	final static float FMNE = 1.0f;

	private static <I extends AbstractBoundedInteger> Integer i_sum(I... i) {
		int sum = IANE;
		for (I n : i) {
			sum += n.value();
		}
		return sum;
	}

	private static <I extends AbstractBoundedInteger> Integer i_sum(List<I> i) {
		int sum = IANE;
		for (I n : i) {
			sum += n.value();
		}
		return sum;
	}

	private static <I extends AbstractBoundedInteger> Integer i_prod(I... i) {
		int prod = IMNE;
		for (I n : i) {
			prod *= n.value();
		}
		return prod;
	}

	private static <I extends AbstractBoundedInteger> Integer i_prod(List<I> i) {
		int prod = IMNE;
		for (I n : i) {
			prod *= n.value();
		}
		return prod;
	}

	private static <F extends AbstractBoundedDouble> Double f_sum(F... f) {
		double sum = FANE;
		for (F n : f) {
			sum += n.value();
		}
		return sum;
	}

	private static <F extends AbstractBoundedDouble> Double f_sum(List<F> f) {
		double sum = FANE;
		for (F n : f) {
			sum += n.value();
		}
		return sum;
	}

	private static <F extends AbstractBoundedDouble> Double f_prod(F... f) {
		double prod = FMNE;
		for (F n : f) {
			prod *= n.value();
		}
		return prod;
	}

	private static <F extends AbstractBoundedDouble> Double f_prod(List<F> f) {
		double prod = FMNE;
		for (F n : f) {
			prod *= n.value();
		}
		return prod;
	}

	public static IHouseholdNumber sum(HouseholdNumber... i) {
		return new HouseholdNumber(f_sum(i));
	}

	public static ISpace sum(Space... i) {
		return new Space(f_sum(i));
	}

	public static IHouseholdNumber prod(HouseholdNumber... i) {
		return new HouseholdNumber(f_prod(i));
	}

	public static ISpace prod(Space... i) {
		return new Space(f_prod(i));
	}

	public static IHouseholdNumber sum(List<HouseholdNumber> i) {
		return new HouseholdNumber(f_sum(i));
	}

	public static ISpace sum(List<Space> i) {
		return new Space(f_sum(i));
	}

	public static IHouseholdNumber prod(List<HouseholdNumber> i) {
		return new HouseholdNumber(f_prod(i));
	}

	public static ISpace prod(List<Space> i) {
		return new Space(f_prod(i));
	}

	public static IHouseholdNumber sum(Collection<HouseholdNumber> i) {
		return new HouseholdNumber(f_sum(i.toArray(new HouseholdNumber[] {})));
	}

	public static ISpace sum(Collection<Space> i) {
		return new Space(f_sum(i.toArray(new Space[] {})));
	}

	public static IHouseholdNumber prod(Collection<HouseholdNumber> i) {
		return new HouseholdNumber(f_prod(i.toArray(new HouseholdNumber[] {})));
	}

	public static ISpace prod(Collection<Space> i) {
		return new Space(f_prod(i.toArray(new Space[] {})));
	}

	public static IPressure sum(Pressure... f) {
		return new Pressure(f_sum(f));
	}

	public static ISatisfaction sum(Satisfaction... f) {
		return new Satisfaction(f_sum(f));
	}

	public static ISupply sum(Supply... f) {
		return new Supply(f_sum(f));
	}

	public static IPressure prod(Pressure... f) {
		return new Pressure(f_prod(f));
	}

	public static ISatisfaction prod(Satisfaction... f) {
		return new Satisfaction(f_prod(f));
	}

	public static ISupply prod(Supply... f) {
		return new Supply(f_prod(f));
	}

	public static IPressure sum(List<Pressure> f) {
		return new Pressure(f_sum(f));
	}

	public static ISatisfaction sum(List<Satisfaction> f) {
		return new Satisfaction(f_sum(f));
	}

	public static ISupply sum(List<Supply> f) {
		return new Supply(f_sum(f));
	}

	public static IPressure prod(List<Pressure> f) {
		return new Pressure(f_prod(f));
	}

	public static ISatisfaction prod(List<Satisfaction> f) {
		return new Satisfaction(f_prod(f));
	}

	public static ISupply prod(List<Supply> f) {
		return new Supply(f_prod(f));
	}

	public static IPressure sum(Collection<Pressure> f) {
		return new Pressure(f_sum(f.toArray(new Pressure[] {})));
	}

	public static ISatisfaction sum(Collection<Satisfaction> f) {
		return new Satisfaction(f_sum(f.toArray(new Satisfaction[] {})));
	}

	public static ISupply sum(Collection<Supply> f) {
		return new Supply(f_sum(f.toArray(new Supply[] {})));
	}

	public static IPressure prod(Collection<Pressure> f) {
		return new Pressure(f_prod(f.toArray(new Pressure[] {})));
	}

	public static ISatisfaction prod(Collection<Satisfaction> f) {
		return new Satisfaction(f_prod(f.toArray(new Satisfaction[] {})));
	}

	public static ISupply prod(Collection<Supply> f) {
		return new Supply(f_prod(f.toArray(new Supply[] {})));
	}

	// public static float prod(Collection<Object> f) {
	// Object[] farray = f.toArray();
	// Type[] ifaces = farray.getClass().getComponentType()
	// .getGenericInterfaces();
	// if (ifaces.length > 0) {
	// for (Type type : ifaces) {
	// switch (type.toString()) {
	// case "ISupply":
	// return TypesUtils.f_prod(f.toArray(new ISupply[] {}));
	// break;
	// case "ISatisfaction":
	// return TypesUtils.f_prod(f.toArray(new ISatisfaction[] {}));
	// break;
	// case "IPressure":
	// return TypesUtils.f_prod(f.toArray(new IPressure[] {}));
	// break;
	// default:
	// return 0;
	// break;
	// }
	// }
	// }
	// }
	//
	// public static int prod(Collection<Object> i) {
	// Object[] farray = i.toArray();
	// Type[] ifaces = farray.getClass().getComponentType()
	// .getGenericInterfaces();
	// if (ifaces.length > 0) {
	// for (Type type : ifaces) {
	// switch (type.toString()) {
	// case "IHouseholdNumber":
	// return TypesUtils.i_prod(i
	// .toArray(new IHouseholdNumber[] {}));
	// break;
	// case "ISpace":
	// return TypesUtils.i_prod(i.toArray(new ISpace[] {}));
	// break;
	// default:
	// return 0;
	// break;
	// }
	// }
	// }
	// }

	public static <T> T prod(Collection<T> n) {
		// Class<? extends INumber> clasz = n.iterator().hasNext()?
		// n.iterator().next().getClass() : null;
		Class<?> clasz = n.iterator().hasNext() ? n.iterator().next()
				.getClass() : null;
		if (clasz.equals(HouseholdNumber.class)) {
			return (T) new HouseholdNumber(f_prod(n
					.toArray(new HouseholdNumber[] {})));
		} else if (clasz.equals(ISpace.class)) {
			return (T) new Space(f_prod(n.toArray(new Space[] {})));
		} else if (clasz.equals(ISupply.class)) {
			return (T) new Supply(f_prod(n.toArray(new Supply[] {})));
		} else if (clasz.equals(ISatisfaction.class)) {
			return (T) new Satisfaction(
					f_prod(n.toArray(new Satisfaction[] {})));
		} else if (clasz.equals(IPressure.class)) {
			return (T) new Pressure(f_prod(n.toArray(new Pressure[] {})));
		} else {
			return null;
		}
	}

	public static <T> T sum(Collection<T> n) {
		// Class<? extends INumber> clasz = n.iterator().hasNext()?
		// n.iterator().next().getClass() : null;
		Class<?> clasz = n.iterator().hasNext() ? n.iterator().next()
				.getClass() : null;
		if (clasz.equals(HouseholdNumber.class)) {
			return (T) new HouseholdNumber(f_sum(n
					.toArray(new HouseholdNumber[] {})));
		} else if (clasz.equals(ISpace.class)) {
			return (T) new Space(f_sum(n.toArray(new Space[] {})));
		} else if (clasz.equals(ISupply.class)) {
			return (T) new Supply(f_sum(n.toArray(new Supply[] {})));
		} else if (clasz.equals(ISatisfaction.class)) {
			return (T) new Satisfaction(f_sum(n.toArray(new Satisfaction[] {})));
		} else if (clasz.equals(IPressure.class)) {
			return (T) new Pressure(f_sum(n.toArray(new Pressure[] {})));
		} else {
			return null;
		}
	}

	// public static float prod(Collection<Object> f) {
	// Object[] farray = f.toArray();
	// Type[] ifaces = farray.getClass().getComponentType()
	// .getGenericInterfaces();
	// if (ifaces.length > 0) {
	// for (Type type : ifaces) {
	// switch (type.toString()) {
	// case "ISupply":
	// return TypesUtils.f_prod(f.toArray(new ISupply[] {}));
	// break;
	// case "ISatisfaction":
	// return TypesUtils.f_prod(f.toArray(new ISatisfaction[] {}));
	// break;
	// case "IPressure":
	// return TypesUtils.f_prod(f.toArray(new IPressure[] {}));
	// break;
	// default:
	// return 0;
	// break;
	// }
	// }
	// }
	// }
	//
	// public static int prod(Collection<Object> i) {
	// Object[] farray = i.toArray();
	// Type[] ifaces = farray.getClass().getComponentType()
	// .getGenericInterfaces();
	// if (ifaces.length > 0) {
	// for (Type type : ifaces) {
	// switch (type.toString()) {
	// case "IHouseholdNumber":
	// return TypesUtils.i_prod(i
	// .toArray(new IHouseholdNumber[] {}));
	// break;
	// case "ISpace":
	// return TypesUtils.i_prod(i.toArray(new ISpace[] {}));
	// break;
	// default:
	// return 0;
	// break;
	// }
	// }
	// }
	// }
}
