/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.common;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

public class Utils {

	/**
	 * FIXME This belongs in core.
	 * 
	 * @param p1
	 * @param message
	 */
	public static <T> void print_debug(Object p1, T message) {

		Utils.print_debug(p1, ""
				+ ((message instanceof String) ? message : message.toString()));
		return;
	}

	public static void print_debug(Object p1, String message) {

		System.err.println(p1.getClass().getSimpleName() + ": " + message);
		return;
	}

	public static synchronized Object evalForeignMethod(Object fObject,
			String methodName) {
		Object back = null;
		try {
			Method invokedMethod = fObject.getClass().getMethod(methodName);
			back = invokedMethod.invoke(fObject);
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return back;
	}

	public static synchronized Object evalForeignMethod(Object fObject,
			String methodName, Class<?>[] parameterTypes, Object... arguments) {
		Object back = null;
		try {
			Method invokedMethod = fObject.getClass().getMethod(methodName,
					parameterTypes);
			back = invokedMethod.invoke(fObject, arguments);
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return back;
	}

	public static synchronized Object evalForeignField(Object fObject,
			String fieldName) {
		Object value = null;
		try {
			value = fObject.getClass().getField(fieldName).get(fObject);
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return value;
	}

	public static synchronized Object evalForeignField(Object fObject,
			String fieldName, Object parameter) {
		Object value = null;
		try {
			fObject.getClass().getField(fieldName).set(fObject, parameter);
			value = evalForeignField(fObject, fieldName);
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return value;
	}

	public static String trim(String input, int startoffset, int endoffset) {
		return String.copyValueOf(input.toCharArray(), startoffset, input
				.length()
				- 1 - endoffset);
	}

	/**
	 * FIXME This belongs to the core.
	 * 
	 * Combines the items in collection by operator
	 * 
	 * @param operator
	 *            One of AND, OR, ...
	 * @param collection
	 *            An object of type {@link Collection}, whose items are to be
	 *            combined.
	 * @return The result of the items' combination
	 */
	public static boolean combine(Operators operator,
			Collection<Boolean> collection) {
		Boolean result = true;
		switch (operator) {
		case AND:
			for (Boolean item : collection) {
				result = result && item;
			}
			break;
		default:
			break;
		}
		return result;
	}

	private static List<Integer> items;

	// = Collections.unmodifiableList(new ArrayList<Integer>(0));

	private static int itemizeStupidly() {
		return items.remove(items.size() - 1);
	}

	public static void initItems() {
		items = new ArrayList<Integer>(Arrays.asList(1,2,3,4,5,6,7,8,9,10,11,12,13));
		Collections.reverse(items);
	}
	
	public Utils() {
		initItems();
	}
	
	public static String itemize() {
		return itemizeStupidly() + ". ";
	}
}
