/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.core.sandbox;

import abmland.types.BoundedFloat;
import abmland.types.Share;
import fj.P1;
import fj.data.Either;

@BoundedFloat(MIN = 0.0f, MAX = 1.0f)
public class ShareDefaultImpl implements Share {

	private static Class<BoundedFloat> inherited_ = BoundedFloat.class;
	private static Class<ShareDefaultImpl> this_ = ShareDefaultImpl.class;

	public final static Float MIN = this_.isAnnotationPresent(inherited_) ? this_
			.getAnnotation(inherited_).MIN()
			: null;

	public final static Float MAX = this_.isAnnotationPresent(inherited_) ? this_
			.getAnnotation(inherited_).MAX()
			: null;

	/** {@inheritDoc} */
	public final Float MAX() {
		return MAX;
	}

	/** {@inheritDoc} */
	public final Float MIN() {
		return MIN;
	}

	static float value;

	public float getLevel() {

		return ShareDefaultImpl.value;
	}

	private static P1<Either<Exception, Float>> eSetVolume(final Float volume) {
		return new P1<Either<Exception, Float>>() {
			/** {@inheritDoc} */
			public Either<Exception, Float> _1() {
				try {
					if (BoundsUtil.isOuterBounds(volume, MIN, MAX)) {
						throw new Exception(volume + " outer bounds");
					}
					ShareDefaultImpl.value = volume;
					return Either.right(ShareDefaultImpl.value);
				} catch (Exception e) {
					return Either.left(e);
				}
			}

		};
	}

	public boolean setLevel(float level) {
		float EDOM = -9999999f;
		P1<Either<Exception, Float>> lazyE = ShareDefaultImpl
				.eSetVolume(level);
		ShareDefaultImpl.value = (lazyE._1().isLeft() ? EDOM : lazyE._1()
				.right().value());
		if (lazyE._1().isLeft()) {
			System.err.println("Failed with " + lazyE._1().left().value());
			return false;
		}
		return true;
	}
}
