/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.core.sandbox;

import fj.P1;
import fj.data.Either;

/**
 * This is the default implementation of the interface describing transfer
 * volume potential regarding units of floating point values.
 * 
 * @author dkahlenb
 * 
 */
public class TransferVolumeFloatsDefaultImpl implements TransferVolumeFloats {

	static float MIN = 0.0f;
	static float MAX = 10000f;
	public static float value = MIN;

	public Float getVolume() {

		return TransferVolumeFloatsDefaultImpl.value;
	}
	
	private static P1<Either<Exception, Float>> eSetVolume(final Float volume) {
		return new P1<Either<Exception, Float>>() {
			@Override
			public Either<Exception, Float> _1() {
				try {
					if (BoundsUtil.isOuterBounds(volume, MIN, MAX)) {
						throw new Exception(volume + " outer bounds");
					}
					TransferVolumeFloatsDefaultImpl.value = volume;
					return Either.right(TransferVolumeFloatsDefaultImpl.value);
				} catch (Exception e) {
					return Either.left(e);
				}
			}

		};
	}
	
	public boolean setVolume(Float volume) {
		float EDOM = -9999999f;
		P1<Either<Exception, Float>> lazyE = TransferVolumeFloatsDefaultImpl.eSetVolume(volume);
		TransferVolumeFloatsDefaultImpl.value = (lazyE._1().isLeft() ? EDOM : lazyE._1().right().value());
		if(lazyE._1().isLeft()) {
			System.err.println("Failed with " + lazyE._1().left().value());
			return false;
		}
		return true;
	}
}
