/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.data;

import abmland.types.IHouseholdNumber;
import abmland.types.IPressure;
import abmland.types.IPriceIndex;
import abmland.types.ISatisfaction;
import abmland.types.ISpace;
import abmland.types.ISupply;

/**
 * @FIXME {p:Paper} Possibly those are candidates for simplexml config too. Or
 *        best remove that completely, using HouseholdNumber.MIN... directly
 *
 * @author dkahlenb
 *
 */
public enum DatatypeBoundaries {

	Satisfaction_LOWER(0.0f, ISatisfaction.class), //
	Satisfaction_UPPER(1.0f, ISatisfaction.class), //
	Space_LOWER(0, ISpace.class), //
	Space_UPPER((int) 1e4, ISpace.class), //
	HouseholdNumber_LOWER(0, IHouseholdNumber.class), //
	HouseholdNumber_UPPER((int) 1e5, IHouseholdNumber.class), //
	Pressure_LOWER(0.0f, IPressure.class), //
	Pressure_UPPER(1.0f, IPressure.class), //
	PriceIndex_LOWER(0.0f, IPriceIndex.class), //
	PriceIndex_UPPER(1.0f, IPriceIndex.class), //
	Supply_LOWER(0.0f, ISupply.class), //
	Supply_UPPER(1e6f, ISupply.class), //
	Boolean_TRUE(1, Boolean.class), //
	Boolean_FALSE(0, Boolean.class);

	private Number value; // as double

	private Class<?> type;

	// private Boolean bool;

	private <T> DatatypeBoundaries(Number value, Class<T> type) {
		this.value = value;
		this.type = type;
	}

	// private DatatypeBoundaries(Boolean value) {
	// this.bool = value;
	// }

	private DatatypeBoundaries() {
		;
	}

	public final Number value() {
		return value;
	}

	public final Class<?> type() {
		return type;
	}

	public final static Class<?> typeOf(DatatypeBoundaries e) {
		return e.type;
	}

	public final static Number valueOf(boolean bool) {
		return bool ? Boolean_TRUE.value() : Boolean_FALSE.value();
	}

	public static void main(String[] args) {
		for (DatatypeBoundaries boundary : values())
			System.out.printf("Boundary for %s is %f%n", boundary, new Double(
					boundary.value().doubleValue()));

		System.err.println(DatatypeBoundaries.valueOf(false));
	}
}
