/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.data;

import static abmland.common.LogExceptionHandler.handleException;

public enum Householdtypes {

	ELDERLYSINGLES(0), ELDERLYCOHABITATIONS(1), FAMILIES(2), FLATSHARERS(3), SINGLEPARENTS(
			4), YOUNGCOHABITATIONS(5), YOUNGSINGLES(6), ELDERLYSINGLES_O(7), ELDERLYCOHABITATIONS_O(
			8), FAMILIES_O(9), FLATSHARERS_O(10), SINGLEPARENTS_O(11), YOUNGCOHABITATIONS_O(
			12), YOUNGSINGLES_O(13)
	// ,UNSPECIFIED (Double.NaN)
	;

	private double value;

	public final double value() {
		return value;
	}

	public final static Householdtypes valueOf(double value) {
		final int intvalue = (int) value;
		for (Householdtypes c : Householdtypes.values()) {
			if ((int) c.value == intvalue)
				return c;
		}
		// if (Double.isNaN(value)) {
		// return Householdtypes.UNSPECIFIED;
		// }
		try {
			throw new IllegalArgumentException(
					"No constant (household type) defined for code "
							+ Double.valueOf(value));
		} catch (Throwable e) {
			handleException(CLCover.class, e);
		}
		return null;
	}

	private Householdtypes() {
		this.value = Double.NaN;
	}

	private Householdtypes(double value) {
		this.value = value;
	}

	public static void main(String[] args) {

		for (Householdtypes ht : Householdtypes.values()) {
			System.err.println(String.format("Constant: %s, Ordinal: %s.", ht.name(), ht.ordinal()));
		}

		System.err.println(Householdtypes.valueOf((int) 1));
	}
}
