/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.mainctrl;

import static abmland.common.CellDumbMock.ALandPriceIndexAllCells.requestLandPriceIndexAllCells;
import static abmland.common.LogExceptionHandler.handleException;
import static abmland.common.RepastUtils.getModelController;
import static abmland.common.ScheduleProperties.*;
import static abmland.models.config.RasterIO.raster2clcmap;
import static abmland.models.config.RasterIO.raster2priceindexmap;
import static org.apache.log4j.Logger.getLogger;
import static org.junit.Assert.assertTrue;

import java.util.Map;

import org.apache.log4j.Logger;

import com.moseph.gis.raster.Raster;

import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.space.grid.GridPoint;
import abmland.data.CLCover;
import abmland.data.Householdtypes;
import abmland.mainctrl.LandUseController.Env;
import abmland.mainctrl.config.ConfManager;
import abmland.mainctrl.config.LanduseConf;
import abmland.mainctrl.config.MarketConf;
import abmland.mainctrl.config.MiscConf;
import abmland.models.IInfrastructureModel;
import abmland.models.ILU;
import abmland.models.IMarket;
import abmland.models.IResidentModel;
import abmland.types.IBooleanAllCells;
import abmland.types.ICLCoverAllCells;
import abmland.types.IFloorSpaceAllCells;
import abmland.types.IHouseholdNumber;
import abmland.types.IHouseholdNumberAllCells;
import abmland.types.ILandPriceIndexAllCells;
import abmland.types.IPriceIndex;

public class CostController implements IMarket {

	private final Logger log = getLogger(getClass().getName());
	private ILU luserver;
	private IFloorSpaceAllCells importedCommercialSpaceAvailability;
	private IHouseholdNumberAllCells importedVacantAccomodation;
	private Env env = new Env();

	final class Env {
		// Use this config reference to retrieve things related to
		// land use, like capacity of residential space for a certain CLC or
		// capacity of business space for a certain CLC and things like that.
		MarketConf config;
		MiscConf miscConfig;
		Map<GridPoint, IPriceIndex> PARAM_INITIAL_LANDPRICE_MAP;
		Map<GridPoint, Float> clcMap;
		private ILandPriceIndexAllCells exportedLandPrice = null;
		private int[] areaDims;
	}

	public CostController() {
		env.config = ConfManager.MARKETCONF.getConfig(MarketConf.class);
		env.miscConfig = ConfManager.MISCCONF.getConfig(MiscConf.class);
		env.areaDims = env.miscConfig.areaParameters.getDims();

		try {
			Raster PARAM_LAND_PRICE_RASTER = env.config.marketParameters
					.getRaster();

			log.trace(PARAM_LAND_PRICE_RASTER.getRows() + ", " + PARAM_LAND_PRICE_RASTER.getCols());

			env.PARAM_INITIAL_LANDPRICE_MAP = raster2priceindexmap(PARAM_LAND_PRICE_RASTER);

			env.exportedLandPrice = requestLandPriceIndexAllCells(PARAM_LAND_PRICE_RASTER, "landprice");

			double w = env.exportedLandPrice.getDimensions().getWidth();
			double h = env.exportedLandPrice.getDimensions().getHeight();

			assertTrue(w == env.areaDims[0] && h == env.areaDims[1]);

			log.trace("CostController w = " + w + " and h = " + h);
			
			log.trace(String.format(
					"testing value at %s, %s (upper right corner patch): %s",
					w, h, (env.exportedLandPrice.get(w - 1, h - 1))));

		} catch (Throwable t) {
			handleException(getClass(), t);
		}
	}

	@ScheduledMethod(//
	start = 1, //
	priority = MAINCTRL_INIT_SCHED_P //
	)
	/** {@inheritDoc} */
	public void init() {
		log.trace(null);
		if (NO_TEST_SCHEDULE) {
			luserver = new LandUseControllerServer(
					getModelController(ILU.class));
			log.info(luserver.getClass().getSimpleName());
		}
		return;
	}

	@ScheduledMethod(//
	start = 2, //
	interval = 1.0, //
	priority = MAINCTRL_GETDATA_SCHED_P //
	)
	/** {@inheritDoc} */
	public void getData() {
		log.trace(null);
		if (NO_TEST_SCHEDULE) {
			importedCommercialSpaceAvailability = luserver
					.getCommercialSpaceAvailability();
			importedVacantAccomodation = luserver.getVacantAccomodation();
		}
		return;
	}

	@ScheduledMethod(//
	start = 2, //
	interval = 1.0, //
	priority = MAINCTRL_COMPUTE_SCHED_P //
	)
	/** {@inheritDoc} */
	public void compute() {
		log.trace(null);
		if (NO_TEST_SCHEDULE) {
			// pause3();
		}
		return;
	}

	/** {@inheritDoc} */
	public ILandPriceIndexAllCells getLandPriceIndex() {
		return env.exportedLandPrice;
	}
}
