/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.mainctrl;

import static org.apache.log4j.Logger.getLogger;

import org.apache.log4j.Logger;

import abmland.data.CLCover;
import abmland.models.ILU;
import abmland.types.ICLCoverAllCells;
import abmland.types.IFloorSpaceAllCells;
import abmland.types.IHouseholdNumberAllCells;

public class LandUseControllerServer implements ILU {

	ILU landuse;

	private final Logger log = getLogger(getClass().getName());

	public LandUseControllerServer(ILU landuse) {
		log.debug("Created with " + landuse.getClass().getSimpleName()
				+ " (id: " + landuse.hashCode() + ")");
		this.landuse = landuse;
	}

	/** {@inheritDoc} */
	public CLCover getScalarLandUse() {
		return landuse.getScalarLandUse();
	}

	/** {@inheritDoc} */
	public IFloorSpaceAllCells getVacantOfficeSpace() {
		return landuse.getVacantOfficeSpace();
	}

//	/** {@inheritDoc} */
//	public IBooleanAllCells getAvailabilityOfMainRoads() {
//		return landuse.getAvailabilityOfMainRoads();
//	}

//	/** {@inheritDoc} */
//	public IBooleanAllCells getAvailabilityOfPublicTransport() {
//		return landuse.getAvailabilityOfPublicTransport();
//	}

	/** {@inheritDoc} */
	public IFloorSpaceAllCells getCommercialSpaceAvailability() {
		return landuse.getCommercialSpaceAvailability();
	}

	/** {@inheritDoc} */
	public ICLCoverAllCells getLandUse() {
		return landuse.getLandUse();
	}

//	/** {@inheritDoc} */
//	public IBooleanAllCells getSchoolSupply() {
//		return landuse.getSchoolSupply();
//	}

	/** {@inheritDoc} */
	public IHouseholdNumberAllCells getVacantAccomodation() {
		return landuse.getVacantAccomodation();
	}

}
