/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.mainctrl;

import static org.apache.log4j.Logger.getLogger;

import org.apache.log4j.Logger;

import repast.simphony.context.DefaultContext;
import abmland.mainctrl.config.ConfManager;
import abmland.models.ILU;
import abmland.models.IMarket;

public class MainControllersContext extends DefaultContext<Object> {

	private final Logger log = getLogger(getClass().getName());

	public MainControllersContext() {

		super("MainControllersContext");

		// First initialise configs so the agents can use them when constructed next
		ConfManager.LANDUSECONF.readConfig();
		ConfManager.MISCCONF.readConfig();
		ConfManager.MARKETCONF.readConfig();

		// Add the agents
		ILU landusectrl = new LandUseController();
		IMarket costctrl = new CostController();
		ScheduleController schedctrl = new ScheduleController();
		ParametersController paramctrl = new ParametersController();

		add(landusectrl);
		add(costctrl);
		add(schedctrl);
		add(paramctrl);

		log.debug("Building " + "MainControllersContext" + " added "
				+ landusectrl.getClass().getSimpleName() + " and "
				+ costctrl.getClass().getSimpleName() + " and "
				+ schedctrl.getClass().getSimpleName() + " and "
				+ paramctrl.getClass().getSimpleName());
	}
}
