/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.mainctrl;

import static abmland.common.LogExceptionHandler.handleException;
import static abmland.common.ScheduleProperties.MODEL_INIT_SCHED_P;
import static org.apache.log4j.Logger.getLogger;
import static org.junit.Assert.assertNotNull;

import org.apache.log4j.Logger;

import abmland.common.Properties;
import abmland.data.CaseStudy;
import abmland.data.DecisionMode;
import abmland.data.Scenario;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduledMethod;

/**
 * Controls the parameters given by extended_params.xml file (Simphony)
 *
 * @author dkahlenb
 *
 */
public class ParametersController {

	static final Logger log = getLogger(ParametersController.class);

	private Env env = new Env();

	private final static class Env {
		static CaseStudy caseStudy;
		static Scenario scenario;
		static int startYear;
		static DecisionMode decisionMode;
	}

	@ScheduledMethod(//
	start = 1, //
	priority = MODEL_INIT_SCHED_P //
	)
	public void init() {
		log.trace(null);

		/**
		 * Get values for variables (user input). They are constant throughout
		 * the simulation.
		 *
		 * E. g. for decision mode each agent has the same decision mode once
		 * chosen (currently is configured with the extended_params.xml file)
		 */
		Env.caseStudy = CaseStudy.valueOf(getYParameters(Properties.CASESTUDY));
		Env.scenario = Scenario
				.valueOf(getYParameters(Properties.SCENARIONAME));
		Env.startYear = Integer.parseInt(getYParameters(Properties.STARTYEAR));
		Env.decisionMode = DecisionMode
				.valueOf(getYParameters(Properties.DECISIONMODE));
	}

	// private String getYParameters(String yParamName) {
	// assert RunEnvironment.getInstance().getParameters() != null;
	// return RunEnvironment.getInstance().getParameters()
	// .getValue(yParamName).toString();
	// }

	/**
	 * @tag usage.general : Assumes that a RunEnvironment is initialised,
	 *      meaning Simphony classes have had to be instrumentised in correct
	 *      manner etc.
	 * @throws NullPointerException
	 *             if the RunEnvironment isn't initialised already.
	 * @param yParamName
	 * @return
	 */
	public static String getYParameters(String yParamName) {
		try {
			assertNotNull(RunEnvironment.getInstance().getParameters());
			return RunEnvironment.getInstance().getParameters().getValue(
					yParamName).toString();
		} catch (Throwable e) {
			handleException(ParametersController.class, e);
		}
		return null;
	}

	public static <T> T getYParameterValue(String yParamName) {
		try {
			assertNotNull(RunEnvironment.getInstance().getParameters());
			return (T) RunEnvironment.getInstance().getParameters().getValue(
					yParamName);
		} catch (Throwable e) {
			handleException(ParametersController.class, e);
		}
		return null;
	}

	public static DecisionMode getDecisionMode() {
		assertNotNull("RunEnvironment not initialised.",RunEnvironment.getInstance());
		assertNotNull("Parameters not yet read.",RunEnvironment.getInstance().getParameters());
		return ParametersController.Env.decisionMode;
	}

	public static CaseStudy getCaseStudy() {
		assertNotNull("RunEnvironment not initialised.",RunEnvironment.getInstance());
		assertNotNull("Parameters not yet read.",RunEnvironment.getInstance().getParameters());
		return ParametersController.Env.caseStudy;
	}

	public static Scenario getScenario() {
		assertNotNull("RunEnvironment not initialised.",RunEnvironment.getInstance());
		assertNotNull("Parameters not yet read.",RunEnvironment.getInstance().getParameters());
		return ParametersController.Env.scenario;
	}

	public static int getStartYear() {
		assertNotNull("RunEnvironment not initialised.",RunEnvironment.getInstance());
		assertNotNull("Parameters not yet read.",RunEnvironment.getInstance().getParameters());
		return ParametersController.Env.startYear;
	}
}
