/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.mainctrl.config;

import static abmland.common.StringUtils.charr2String;
import static abmland.mainctrl.ParametersController.getYParameters;
import static java.lang.System.getProperty;
import static org.junit.Assert.assertTrue;
import abmland.common.Properties;
import abmland.models.config.CohortConf;
import abmland.models.config.CohortPhiConf;
import abmland.models.config.ConfigReaderImpl;
import abmland.models.config.InfrastructureConf;
import abmland.models.config.MapsConf;
import abmland.models.config.ModelConfig;
import abmland.models.config.ResidentConf;

public enum ConfManager {

	MARKETCONF(new ConfigReaderImpl(), MarketConf.class)//
	, INFRASTRUCTURECONF(new ConfigReaderImpl(), InfrastructureConf.class)//
	, COHORTCONF(new ConfigReaderImpl(), CohortConf.class)//
	, COHORTPHICONF(new ConfigReaderImpl(), CohortPhiConf.class)//
	, RESIDENTCONF(new ConfigReaderImpl(), ResidentConf.class)//
	, MAPSCONF(new ConfigReaderImpl(), MapsConf.class)//
	, MISCCONF(new ConfigReaderImpl(), MiscConf.class)//
	, LANDUSECONF(new ConfigReaderImpl(), LanduseConf.class);

	// @SuppressWarnings("unchecked")
	private <T> ConfManager(ConfigReaderImpl reader, Class<T> modelConfigClazz
	// , String... configBasePath
	) {
		this.modelConfigClazz = modelConfigClazz;
		this.reader = reader;
		// this.config = this.reader.readConfig(charr2String("config"
		// + getProperty("file.separator") + "params", configBasePath)
		// // using a static configuration scheme for a start...
		// + getProperty("file.separator")
		// + getYParameters(Properties.CASESTUDY)
		// + getProperty("file.separator")
		// + getYParameters(Properties.STARTYEAR)
		// + getProperty("file.separator")
		// + getYParameters(Properties.SCENARIONAME),
		// this.modelConfigClazz);
	}

	private ModelConfig config = null;
	@SuppressWarnings("unchecked")
	private Class modelConfigClazz = null;
	private ConfigReaderImpl reader = null;

	@SuppressWarnings("unchecked")
	public <T extends ModelConfig> T getConfig() {
		assertTrue("Please first call readConfig.", null != this.config);
		return (T) config;
	}

	public <T extends ModelConfig> T getConfig(Class<T> clazz) {
		assertTrue("Please first call readConfig.", null != this.config);
		T c = clazz.cast(config);
		return c;
	}

	/**
	 * Is for reading config from an optionally given (default is config/params)
	 * custom path simply, no sub-folders, config xml files directly at path.
	 * Doesn't assume that Simphonies RunEnvironment is initialised, useful for
	 * the case of (Simphony) contexts constructors/at construction time...
	 *
	 * @param configBasePath
	 */
	@SuppressWarnings("unchecked")
	public void readPureConfig(String... configBasePath) {
		assertTrue("ConfManager initialisation error.", null != this.reader
				&& null != this.modelConfigClazz);
		this.config = this.reader.readConfig(charr2String("config"
				+ getProperty("file.separator") + "params", configBasePath)
		// using a static configuration scheme for a start...
				, this.modelConfigClazz);
	}

	/**
	 * Is for reading config from an optionally given (default is config/params)
	 * custom path containing the CASESTUDY.STARTYEAR.SCENARIONAME file name
	 * scheme. Assumes that Simphonies RunEnvironment is initialised, which
	 * isn't the case in the (Simphony) contexts constructors/at construction
	 * time... For config needed there use the readPureConfig implementation
	 *
	 * @param configBasePath
	 */
	@SuppressWarnings("unchecked")
	public void readConfig(String... configBasePath) {
		assertTrue("ConfManager initialisation error.", null != this.reader
				&& null != this.modelConfigClazz);
		this.config = this.reader.readConfig(charr2String("config"
				+ getProperty("file.separator") + "params", configBasePath)
				// using a static configuration scheme for a start...
				+ getProperty("file.separator")
				+ getYParameters(Properties.CASESTUDY)
				+ getProperty("file.separator")
				+ getYParameters(Properties.STARTYEAR)/*
				+ getProperty("file.separator")
				+ getYParameters(Properties.SCENARIONAME)*/,
				this.modelConfigClazz);
	}

	@SuppressWarnings("unchecked")
	public void readMapsConfig(String... configBasePath) {
		assertTrue("ConfManager initialisation error.", null != this.reader
				&& null != this.modelConfigClazz);
		this.config = this.reader.readConfig(charr2String("config"
				+ getProperty("file.separator") + "maps", configBasePath)
				// using a static configuration scheme for a start...
				+ getProperty("file.separator")
				+ getYParameters(Properties.CASESTUDY)
				+ getProperty("file.separator")
				+ getYParameters(Properties.STARTYEAR), this.modelConfigClazz);
	}

	// public void setConfScheme() {
	//
	// }
}
