/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.mainctrl.config;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import abmland.data.CLCover;
import abmland.models.config.RasterIO;
import abmland.models.config.RasterTemplate;
import com.moseph.gis.raster.Raster;
import org.simpleframework.xml.Attribute;
import org.simpleframework.xml.ElementList;
import org.simpleframework.xml.core.Commit;

public class LanduseParameters {

	@Attribute(required = false)
	public String clcAscii;
	private RasterTemplate asciiFile;

	@ElementList
	private List<CLC> clcProperties = new ArrayList<CLC>();
	private Map<CLCover, CLC> templateMap = new HashMap<CLCover, CLC>();

	public Raster getRaster() {
		return RasterIO.buildRaster(this.asciiFile.filename);
	}

	public Entry<CLCover, CLC> getProperties(CLCover clc) {
		for (Entry<CLCover, CLC> e : templateMap.entrySet()) {
			return e.getKey() == clc ? e : null;
		}
		return null;
	}

	public int getResidentialCapacity(CLCover clc) {
		return /** getProperties(clc).getValue().residentialCapacity; */
		templateMap.get(clc).residentialCapacity;
	}

	public int getCommercialCapacity(CLCover clc) {
		return /** getProperties(clc).getValue().commercialCapacity; */
		templateMap.get(clc).commercialCapacity;
	}

	@Commit
	public void onCommit() {
		this.asciiFile = new RasterTemplate();

		this.asciiFile.filename = this.clcAscii;

		for (CLC t : clcProperties)
			templateMap.put(t.id, t);
	}
}