/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.models;

import java.util.Map;
import java.util.Set;

import repast.simphony.space.grid.GridPoint;
import abmland.data.Businesstypes;
import abmland.data.CLCover;
import abmland.data.Issuetypes;
import abmland.types.IBooleanAllCells;
import abmland.types.IBusinesstypesAllCells;
import abmland.types.IFloorSpaceAllCells;
import abmland.types.ISatisfaction;

public interface IBusinessModel {
	/**
	 * @return a full spatial distribution of needed square meters office space
	 *         each cell.
	 */
	IFloorSpaceAllCells getCommercialSpaceDemand();

	/**
	 * @return a spatial distribution of satisfaction per issue mapped to
	 *         different business types each enclosed cell
	 */
	Map<GridPoint, Map<Businesstypes, Map<Issuetypes, ISatisfaction>>> getSatisfaction();

	/**
	 * XXX Original req. was: return a spatial distribution of supply/demand
	 * shares for goods per cell.
	 *
	 * @return a spatial distribution of 0..many business types each enclosed
	 *         cell
	 */
	IBusinesstypesAllCells getBusinessPresence();

	/**
	 * @return a full spatial distribution of binary decisions about the need
	 *         for infrastructure.
	 */
	IBooleanAllCells getInfrastructureDemand();

	/**
	 * @return a spatial distribution of potentially new CLC each enclosed cell
	 */
	Map<GridPoint, CLCover> getLandUseChanges();

	/**
	 * @return a set of proposals (proposal meaning spatial distribution of CLC)
	 */
	Set<Map<GridPoint, CLCover>> getProposalsNeedingPermission();

	/**
	 * @return a full spatial distribution of space units occupied each cell
	 */
	IFloorSpaceAllCells getCommercialSpaceOccupation();
}
