/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.models;

import java.util.Map;
import java.util.Set;

import repast.simphony.space.grid.GridPoint;
import abmland.data.CLCover;
import abmland.types.IBooleanAllCells;

public interface IInfrastructureModel {
	/**
	 * Exports a map of CLC wished to be permitted per cell/cell aggregation.
	 * Means that multiple cells aggregated may be regarded each as commutative
	 * region for which one certain targeted land use is requested to be
	 * permitted. Or in short terms, it wraps Map&lt;CellSpec..., CLCover&gt; as
	 * you can see in IPermissionsMap. <br>
	 *
	 * @return a set of spatial distributions of CLC (proposals)
	 */
	Set<Map<GridPoint, CLCover>> getProposalsNeedingPermission();

	// /**
	// * Exports a map of supply/demand shares for infrastructure per cell to
	// * {@link IBusinessModel}.
	// *
	// * @return a map of supply/demand shares for infrastructure per cell.
	// */
	// ISupplyAllCells getInfrastructureSupply();

	/**
	 * @return a spatial distributions of potentially new CLC
	 */
	Map<GridPoint, CLCover> getLandUseChanges();

	/**
	 * FIXME Dagmar: Shure works with a threshold value (?), i. e. school
	 * (school type) more than 5 min. by car/public transport or so leads to
	 * availability = FALSE for a cell?? <br>
	 *
	 * @return a full spatial distribution of binary decisions on the supply of
	 *         schools
	 */
	IBooleanAllCells getSchoolSupply();

	/**
	 * FIXME Dagmar: Shure works with a threshold value (?), i. e. main road
	 * more than 5 min. by car or so leads to availability = FALSE for a cell?? <br>
	 *
	 * @return a full spatial distribution of binary decisions on available main
	 *         roads
	 */
	IBooleanAllCells getAvailabilityOfMainRoads();

	/**
	 * FIXME Dagmar: Shure works with a threshold value (?), i. e. public
	 * transport more than 5 min. per feet leads to availability = FALSE for a
	 * cell?? <br>
	 *
	 * @return a full spatial distribution of binary decisions on available
	 *         public transport
	 */
	IBooleanAllCells getAvailabilityOfPublicTransport();
}
