/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.models;

import abmland.data.CLCover;
import abmland.types.ICLCoverAllCells;
import abmland.types.IFloorSpaceAllCells;
import abmland.types.IHouseholdNumberAllCells;

public interface ILU {

	/**
	 * @return a CLCover representing the last step's LU by convention
	 *
	 * @FIXME {p:Paper} Needs further clarification, what is meant.
	 */
	CLCover getScalarLandUse();

	/**
	 * Current idea of implementation: The land uses requested by
	 * infrastructure, developer and business are drawn from some informed-guess
	 * priority-score and then an updated overall land use map is export-able
	 * per step.
	 *
	 * @return
	 */
	ICLCoverAllCells getLandUse();

	/**
	 * land use can export an all cells object with the capacity of available
	 * commercial space, this is computed in simplest terms as a ratio of
	 * commercial space occupied exported by business - and commercial capacity
	 * which we assume to be a property related to corine land cover a case
	 * study area patch has (is given by configuration in current
	 * implementation, see LanduseConf.java).
	 *
	 * Current idea of implementation: While in the get data phase, land use
	 * ctrl queries the business agent for needed dynamic data (occupied
	 * commercial space) to compute the ratio.
	 *
	 * @return the re-organised object with the designated space distribution on
	 *         all cells
	 */
	IFloorSpaceAllCells getVacantOfficeSpace();

	/**
	 * land use can export an all cells object with the capacity of available
	 * commercial space, this is computed in simplest terms as a ratio of
	 * commercial space occupied exported by business - and commercial capacity
	 * which we assume to be a property related to corine land cover a case
	 * study area patch has (is given by configuration in current
	 * implementation, see LanduseConf.java).
	 *
	 * Current idea of implementation: While in the get data phase, land use
	 * ctrl queries the business agent for needed dynamic data (occupied
	 * commercial space) to compute the ratio.
	 *
	 * @return the re-organised object with the designated space distribution on
	 *         all cells
	 */
	IFloorSpaceAllCells getCommercialSpaceAvailability();

	// XXX done by infrastructure itself already
	// /**
	// * land use can export an all cells object with the availability of
	// schools,
	// * this is what infrastructure computes.
	// *
	// * Current idea of implementation: While in the get data phase, land use
	// * ctrl queries the infrastructure agent for that data.
	// *
	// * @return
	// */
	// IBooleanAllCells getSchoolSupply();

	// XXX done by infrastructure itself already
	// /**
	// * land use can export an all cells object with the availability of main
	// * roads, this is what infrastructure computes.
	// *
	// * Current idea of implementation: While in the get data phase, land use
	// * ctrl queries the infrastructure agent for that data.
	// *
	// * @return
	// */
	// IBooleanAllCells getAvailabilityOfMainRoads();

	// XXX done by infrastructure itself already
	// /**
	// * land use can export an all cells object with the availability of
	// * transport, this is what infrastructure computes.
	// *
	// * Current idea of implementation: While in the get data phase, land use
	// * ctrl queries the infrastructure agent for that data.
	// *
	// * @return
	// */
	// IBooleanAllCells getAvailabilityOfPublicTransport();

	/**
	 * land use can export an all cells object with the capacity of vacant
	 * accomodations, this is computed in simplest terms as a ratio of total
	 * household number exported by resident - and residential capacity which we
	 * assume to be a property related to corine land cover a case study area
	 * patch has (is given by configuration in current implementation, see
	 * LanduseConf.java).
	 *
	 * Current idea of implementation: While in the get data phase, land use
	 * ctrl queries the resident agent for needed dynamic data (household
	 * number) to compute the ratio.
	 *
	 * @return the re-organised object with the designated household number
	 *         distribution on all cells
	 */
	IHouseholdNumberAllCells getVacantAccomodation();
}