/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.models;

import java.util.Map;
import java.util.Set;

import repast.simphony.space.grid.GridPoint;
import abmland.data.CLCover;
import abmland.types.ICLCoversAllCells;
import abmland.types.IPressure;

public interface IPlannerModel {

	/**
	 *
	 * @return a table mapping binary decisions to spatial distributions of CLC (proposals)
	 */
	Map<Map<GridPoint, CLCover>, Boolean> getPermissionForProposals();

	/**
	 *
	 * @return a spatial distribution mapping pressure to certain CLC each enclosed cell
	 */
	Map<GridPoint, Map<CLCover, IPressure>> getSpatialPressure();

	/**
	 *
	 * @return a table mapping pressure to certain CLC
	 */
	Map<CLCover, IPressure> getGlobalPressure();

	/**
	 *
	 * @return a set of proposals (spatial distributions of clc)
	 */
	Set<Map<GridPoint, CLCover>> getPendingProposals();

	/**
	 *
	 * @return a spatial distribution of 0..many CLC each enclosed cell
	 */
	ICLCoversAllCells getPermittedLandUse();
}
