/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.models.config;

import java.util.EnumMap;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.simpleframework.xml.ElementMap;

import abmland.data.Householdtypes;
import abmland.data.Populationclazzes;

public class CohortPhiConf implements ModelConfig {

	@ElementMap
	public Map<Householdtypes, PhiPerCohort> residents = new EnumMap<Householdtypes, PhiPerCohort>(
			Householdtypes.class);

	// @Override
	public String getModelConfigName() {
		return "CohortPhiConf";
	}

	public void put(Householdtypes key, PhiPerCohort value) {
		residents.put(key, value);
	}

	/**
	 * Use getPhiMatrixEntries().
	 *
	 * @return a set with entries mapping the phi-value-to-cohort configurations to the
	 *         distinct household types as a whole
	 */
	public Set<Entry<Householdtypes, PhiPerCohort>> dumpEntries() {
		return residents.entrySet();
	}

	/**
	 *
	 * @return a set with entries mapping the phi-value-to-cohort configurations to the
	 *         distinct household types as a whole
	 */
	public Set<Entry<Householdtypes, PhiPerCohort>> getPhiMatrixEntries() {
		return residents.entrySet();
	}

	/**
	 * Use getPhiMatrixEntries(Householdtypes hdtype).
	 *
	 * @param hdtype
	 *            - the household type in question
	 * @return a map with the phi values directly mapped to cohorts for the
	 *         current household type
	 */
	public Set<Entry<Populationclazzes, Float>> dumpPhiPerCohort(
			Householdtypes hdtype) {
		return getPhiMatrixEntries(hdtype);
	}

	/**
	 * @param hdtype
	 *            - the household type in question
	 * @return a map with the phi values directly mapped to cohorts for the
	 *         current household type
	 */
	public Set<Entry<Populationclazzes, Float>> getPhiMatrixEntries(
			Householdtypes hdtype) {
		return residents.get(hdtype).dumpPhiPerCohort();
	}
}
