/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.models.config;

import static abmland.common.LogExceptionHandler.handleException;
import static org.apache.log4j.Logger.getLogger;
import static org.junit.Assert.fail;

import java.io.File;
import java.net.URISyntaxException;

import org.apache.log4j.Logger;
import org.simpleframework.xml.core.Persister;

public class ConfigIO {

	private static final Logger log = getLogger(ConfigIO.class);

	public static <C> C read(Persister p, String configfilename, C config) {
		File f = locate(configfilename);
		try {
			Object readin = p.read(config.getClass(), f);
			/**
			 * XXX `config.getClass() == C` -> so casting is allowed, see
			 * enclosing method's parameter `config`
			 */
			config = readin.getClass() == config.getClass() ? (C) readin : null;
		} catch (Throwable t) {
			Object readin = null;
			try {
				readin = p.read(config.getClass(), ConfigIO.class
						.getClassLoader().getResourceAsStream(
								f.getAbsolutePath()));
			} catch (Exception e) {
				handleException(ConfigIO.class, e);
			}
			/**
			 * XXX `config.getClass() == C` -> so casting is allowed, see
			 * enclosing method's parameter `config`
			 */
			config = readin.getClass() == config.getClass() ? (C) readin : null;
			handleException(ConfigIO.class, t);
		}
		return config;
	}

	public static <C> void write(Persister persister, String configfilename,
			C config) {
		try {
			persister.write(config, new File(configfilename));
		} catch (Throwable t) {
			t.printStackTrace();
			fail();
		}
	}

	/**
	 * FIXME Put this into ConfigIO
	 *
	 * @param resource
	 * @return
	 */
	public static File locate(String resource) {
		try {
			// return new File(ConfigIO.class.getClassLoader().getResource(
			// resource).toURI());
			File f = new File(resource);
			if (!f.exists())
				return new File(ConfigIO.class.getClassLoader().getResource(
						resource).toURI());
			return f;
			// for stream then use
			// XX.class.getClassLoader().getResourceAsStream(result.getAbsolutePath())
		} catch (URISyntaxException e) {
			handleException(ConfigIO.class, e);
			return null;
		}
	}
}
