/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.models.config;

import static abmland.common.LogExceptionHandler.handleException;
import static abmland.models.config.RasterIO.buildRaster;
import static org.apache.log4j.Logger.getLogger;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.log4j.Logger;
import org.simpleframework.xml.Attribute;
import org.simpleframework.xml.core.Commit;


import com.moseph.gis.raster.Raster;

public class HouseholdtypeParameters {

	@Attribute(required = false)
	public Float dummyFloat;

	@Attribute(required = false)
	public String yesNoAscii;

	private Raster raster;

	public HouseholdtypeParameters() {
		try {
			raster = new Raster();
		} catch (Throwable e) {
			handleException(getClass(), e);
		}
	}

	private RasterTemplate asciiFile;

	@Commit
	public void onCommit() {
		// log.debug(((Number) Float.parseFloat(Float.NaN + "")).floatValue());
		// log.debug(Float.parseFloat("NaN"));
		asciiFile = new RasterTemplate();

		asciiFile.filename = yesNoAscii;

		for (Entry<String, Object> configValue : configValues.entrySet()) {
			log.trace(String.format("Storing %s into %s", configValue
					.getValue(), configValue.getKey()));
			setField(configValue.getKey(), configValue.getValue());
		}
	}

	private Map<String, Object> configValues = new HashMap<String, Object>();

	public void putConfigValue(String name, Object value) {
		configValues.put(name, value);
		setField(name, configValues.get(name));
	}

	private final Logger log = getLogger(getClass().getName());

	private void setField(String name, Object value) {
		log.trace(String.format("Will try to store %s with value %s", name,
				value));
		try {
			Field t = getClass().getField(name);
			log.trace(String.format("Stored %s with value %s", name, value));
			t.set(this, value);
			return;
		} catch (Throwable e) {
			handleException(getClass(), e);
		}
		log.error(String.format("%s with value %s not stored!", name, value));
	}

	/**
	 * @return a raster template built from the file given in the xml file by
	 *         yesNoAscii
	 */
	public Raster getRaster() {
		this.raster = buildRaster(asciiFile.filename);
		return raster;
	}

	public int[] getRasterDimensions() {
		if (raster.getRows() == 0 && raster.getCols() == 0)
			log
					.info("Empty raster. Raster not yet built. Prior call getRaster() to get raster from real file");

		return new int[] { raster.getRows(), raster.getCols() };
	}
}
