/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.models.config;

import static org.junit.Assert.assertNotNull;
import static abmland.models.config.RasterIO.buildRaster;
import static abmland.common.LogExceptionHandler.handleException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.simpleframework.xml.ElementList;
import org.simpleframework.xml.core.Commit;

import com.moseph.gis.raster.Raster;

public class MapsConf implements ModelConfig {

	@ElementList
	private List<RasterTemplate> maps = new ArrayList<RasterTemplate>();

	private Set<RasterTemplate> templates = new HashSet<RasterTemplate>();

	public Map<String, RasterTemplate> rasters = new HashMap<String, RasterTemplate>();

	// @Override
	public String getModelConfigName() {
		return "MapsConf";
	}

	public void add(RasterTemplate tpl) {
		maps.add(tpl);
	}

	@Commit
	public void onCommit() {
		for (RasterTemplate t : maps) {
			if (templates.add(t)) {
				rasters.put(t.name, t);
			}
		}
	}

	public Set<Entry<String, RasterTemplate>> dumpEntries() {
		return rasters.entrySet();
	}

	public Raster getRaster(String name) {
		try {
			assertNotNull(rasters.get(name));
			return buildRaster(rasters.get(name).filename);
		} catch (Throwable e) {
			handleException(getClass(), e);
		}
		return null;
	}
}
