/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.models.config;

import static abmland.common.LogExceptionHandler.handleException;
import static abmland.common.Properties.CASESTUDY;
import static abmland.common.Properties.STARTYEAR;
import static abmland.mainctrl.ParametersController.getYParameters;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URL;

import org.apache.log4j.Logger;

import com.moseph.gis.raster.Raster;
import com.moseph.gis.raster.RasterReader;

/**
 * TODO Do we need the class loader stuff here too?
 *
 * @author dkahlenb
 *
 */
public class RasterReaderWrapper {

	private static Logger log = Logger.getLogger(RasterReaderWrapper.class);

	/**
	 * TODO How to make the System.getProperty("file.separator"); work
	 */
	private final static String DS = "/";

	static String mapsConfig = "config" + DS + "maps";

	public Raster readRaster(String pathTo, String rasterBasename)
			throws Exception {
		String rasterFilename = pathTo + DS + rasterBasename;
		// ClassLoader cll = Thread.currentThread().getContextClassLoader();
		ClassLoader cll = getClass().getClassLoader();

		// (1) Try to read as file...
		try {
			// [debug] throws FileNotFoundException
			URL rasterFileUrl = cll.getResource(rasterFilename);
			assertNotNull("resource is null", rasterFileUrl);
			rasterFilename = new File(rasterFileUrl.getFile()).getCanonicalPath()
					.replaceAll("\\%20", " ");

			// [debug] throws FileNotFoundException
			new FileInputStream(rasterFilename);

			log.trace("Delegating to read per file (" + rasterFilename
					+ " found).");

		} catch (Throwable t) {
			handleException(getClass(), t);
			fail();
		} finally {
			log.trace("Reading raster from file: " + rasterFilename);
		}

		// first try with basename
//		try {
//			URL fileurl = cll.getSystemResource(rasterFilename);
//			assertNotNull("resource not found.", fileurl);
//			// throws IOException
//			fileurl.openStream();
//			rasterFilename = new File(fileurl.getFile()).getCanonicalPath()
//					.replaceAll("\\%20", " ");
//		} catch (Throwable t1) {
//			handleException(getClass(), t1);
//		} finally {
//			log.trace("Reading raster from file: " + rasterFilename);
//		}
		return getMosephRaster(rasterFilename);
	}

	/**
	 * TODO Do we need a case here, where the user already specified an absolute
	 * file name (with directory part) instead of base name only?!
	 *
	 * @param rasterBasename
	 * @return the read in raster
	 * @throws Exception
	 */
	public Raster readRaster(String rasterBasename) throws Exception {

		String caseStudy = getYParameters(CASESTUDY);
		String startYear = getYParameters(STARTYEAR);
		// wanted path: maps/CASESTUDY/STARTYEAR/file

		return readRaster(mapsConfig + DS + caseStudy + DS + startYear,
				rasterBasename);
	}

	private Raster getMosephRaster(String filename) throws IOException {
		return new RasterReader().readRaster(filename);
	}
}
