/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.models.default_impl;

import static abmland.common.CellDumbMock.ABooleanAllCells.requestBooleanAllCells;
import static abmland.common.CellDumbMock.ACLCoverSomeCells.requestCLCoverSomeCells;
import static abmland.common.CellDumbMock.AProposalsSet.requestProposalsSet;
import static abmland.common.ScheduleProperties.DECISION_MODE_IS_STATIC_COND;
import static abmland.common.ScheduleProperties.MODEL_GETDATA_SCHED_P;
import static abmland.common.ScheduleProperties.MODEL_INIT_SCHED_P;
import static org.apache.log4j.Logger.getLogger;

import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.engine.watcher.Watch;
import repast.simphony.engine.watcher.WatcherTriggerSchedule;
import repast.simphony.space.grid.GridPoint;
import abmland.data.CLCover;
import abmland.models.IInfrastructureModel;
import abmland.models.config.ModelConfig;
import abmland.types.IBooleanAllCells;

public class DefaultInfrastructureModel2 implements IInfrastructureModel,
		ModelConfig {

	private static final String DEFAULTTRACEMSG = DefaultInfrastructureModel2.class
	.getCanonicalName();
	private static final String SCHEDULECTRL = "abmland.mainctrl.ScheduleController";
	private static final String CHECK = "checkDSmode";
	private final Logger log = getLogger(getClass().getName());

	/** {@inheritDoc} */ public IBooleanAllCells getAvailabilityOfMainRoads() {
		return requestBooleanAllCells("Mainroads");
	}

	/** {@inheritDoc} */ public IBooleanAllCells getAvailabilityOfPublicTransport() {
		return requestBooleanAllCells("PublicTP");
	}

	/** {@inheritDoc} */ public Map<GridPoint, CLCover> getLandUseChanges() {
		return requestCLCoverSomeCells();
	}

	/** {@inheritDoc} */ public Set<Map<GridPoint, CLCover>> getProposalsNeedingPermission() {
		return requestProposalsSet(null);
	}

	/** {@inheritDoc} */ public IBooleanAllCells getSchoolSupply() {
		return requestBooleanAllCells("Schools");
	}

	/**
	 * What to do at the `init` phase. Triggered by Simphony scheduling RTS
	 */
	@ScheduledMethod(//
	start = 1, //
	priority = MODEL_INIT_SCHED_P //
	)
	public void init() {
		log.trace(DEFAULTTRACEMSG);
	}

	/**
	 * What to do at the `get data` phase. Triggered by Simphony scheduling RTS
	 */
	@ScheduledMethod(//
	start = 2, //
	interval = 1.0, //
	priority = MODEL_GETDATA_SCHED_P //
	)
	public void getData() {
		log.trace(DEFAULTTRACEMSG);
	}

	/** Static `decision mode` implementation. Triggered by Simphony watching query RTS */
	@Watch(//
	watcheeClassName = SCHEDULECTRL, //
	watcheeFieldNames = CHECK, //
	whenToTrigger = WatcherTriggerSchedule.IMMEDIATE, //
	scheduleTriggerDelta = 1, //
	triggerCondition = DECISION_MODE_IS_STATIC_COND, //
	scheduleTriggerPriority = ScheduleParameters.FIRST_PRIORITY)
	public void computeSTATIC() {
		log.trace(DEFAULTTRACEMSG);
	}

	/** Clever `decision mode` implementation. Triggered by Simphony watching query RTS */
	@Watch(//
	watcheeClassName = SCHEDULECTRL, //
	watcheeFieldNames = CHECK, //
	whenToTrigger = WatcherTriggerSchedule.IMMEDIATE, //
	scheduleTriggerDelta = 1, //
	triggerCondition = DECISION_MODE_IS_STATIC_COND, //
	scheduleTriggerPriority = ScheduleParameters.FIRST_PRIORITY)
	public void computeCLEVER() {
		log.trace(DEFAULTTRACEMSG);
	}

	/** Random `decision mode` implementation. Triggered by Simphony watching query RTS */
	@Watch(//
	watcheeClassName = SCHEDULECTRL, //
	watcheeFieldNames = CHECK, //
	whenToTrigger = WatcherTriggerSchedule.IMMEDIATE, //
	scheduleTriggerDelta = 1, //
	triggerCondition = DECISION_MODE_IS_STATIC_COND, //
	scheduleTriggerPriority = ScheduleParameters.FIRST_PRIORITY)
	public void computeRANDOM() {
		log.trace(DEFAULTTRACEMSG);
	}

	/** {@inheritDoc} */ public String getModelConfigName() {
		return "DefaultInfrastructureModel2";
	}
}
