/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.models.default_impl;

import static abmland.common.LogExceptionHandler.handleException;
import static org.apache.log4j.Logger.getLogger;

import java.util.Iterator;

import org.apache.log4j.Logger;

import repast.simphony.context.DefaultContext;
import abmland.data.Householdtypes;
import abmland.mainctrl.config.ConfManager;
import abmland.models.config.ResidentConf;

public class ResidentAgentContext extends DefaultContext<Object> {

	private final Logger log = getLogger(getClass().getName());

	public ResidentAgentContext() {
		super(ResidentAgentContext.class.getName());

		ConfManager.RESIDENTCONF.readPureConfig();
		ResidentConf residentConf = ConfManager.RESIDENTCONF.getConfig();
		String RESIDENT_AGENT = residentConf.getModelClassName();

		try {
			Class<?> agentClazz = Class.forName(RESIDENT_AGENT);
			for (Object _ : Householdtypes.values()) {
				add(agentClazz.newInstance());

				Iterator<?> i = getObjects(agentClazz).iterator();
				Object o = null;
				while(i.hasNext()) {
					o = i.next();
				}
				log.trace("Added " + agentClazz.getMethod("toString").invoke(o)
						+ " to built " + ResidentAgentContext.class.getName());
				log.trace("A little test:");
//				 Assumes not enforces, that the agent implementor gave his agent an init method
				agentClazz.getMethod("init").invoke(o);
			}

		} catch (Throwable t) {
			handleException(ResidentAgentContext.class, t);
		}
	}
}
