/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.models.default_impl;

import static abmland.common.LogExceptionHandler.handleException;
import static abmland.common.Properties.RESIDENT_MODEL_CONTEXT;
import static org.apache.log4j.Logger.getLogger;

import org.apache.log4j.Logger;

import repast.simphony.context.DefaultContext;
import abmland.mainctrl.config.ConfManager;
import abmland.models.IResidentModel;
import abmland.models.config.ResidentConf;

public class ResidentModelContext extends DefaultContext<IResidentModel> {

	private final Logger log = getLogger(getClass().getName());

	public ResidentModelContext() {

		// can give principally anything as String here, may even leave
		// constructor call parameters empty
		super(ResidentModelContext.class.getName());
		ConfManager.RESIDENTCONF.readPureConfig();
		ResidentConf residentConf = ConfManager.RESIDENTCONF.getConfig();
		String RESIDENT_MODEL = residentConf.getModelClassName();
		// super();
		// using some reflection here
		try {
			Class<IResidentModel> modelClazz = (Class<IResidentModel>) Class
					.forName(RESIDENT_MODEL);
			IResidentModel model = modelClazz.newInstance();
			add(model);
			log.trace("Building " + RESIDENT_MODEL_CONTEXT + " added "
					+ model.getClass().getSimpleName());
		} catch (ClassNotFoundException e) {
			log.fatal("No class " + RESIDENT_MODEL
					+ " for instantiation found.");
			handleException(ResidentModelContext.class, e);
		} catch (InstantiationException e) {
			log
					.fatal("Given model class isn't instantiateable for other reasons (e. g. is abstract)??");
			handleException(ResidentModelContext.class, e);
		} catch (IllegalAccessException e) {
			log
					.fatal("Given model class seems to have a private constructor??");
			handleException(ResidentModelContext.class, e);
		} catch (ClassCastException e) {
			log.fatal("Given model doesn't implement " + IResidentModel.class);
			handleException(ResidentModelContext.class, e);
		}
		// IResidentModel modelAgent = new DefaultResidentModel2();
		// add(modelAgent);
		// log.debug("Building " + RESIDENT_MODEL_CONTEXT + " added "
		// + modelAgent.getClass().getSimpleName());
	}
}
