/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.types;

import java.lang.annotation.Annotation;

@BoundedFloat
public abstract class AbstractBoundedFloat extends Number implements
/** BoundedFloat, */
Comparable<AbstractBoundedFloat> {

	/** {@inheritDoc} */
	public double doubleValue() {
		return Float.valueOf(value()).doubleValue();
	}

	/** {@inheritDoc} */
	public float floatValue() {
		return Float.valueOf(value()).floatValue();
	}

	/** {@inheritDoc} */
	public int intValue() {
		return Float.valueOf(value()).intValue();
	}

	/** {@inheritDoc} */
	public long longValue() {
		return Float.valueOf(value()).longValue();
	}

	/** {@inheritDoc} */
	public abstract float MAX();

	/** {@inheritDoc} */
	public abstract float MIN();

	/** {@inheritDoc} */
	public abstract float NODATA();

	/** {@inheritDoc} */
	public abstract float value();

	/** {@inheritDoc} */
	public Class<? extends Annotation> annotationType() {
		return BoundedFloat.class.getAnnotation(BoundedFloat.class)
				.annotationType();
	}

	/** {@inheritDoc} */
	public int compareTo(AbstractBoundedFloat o) {
		return Float.valueOf(value()).compareTo(Float.valueOf(o.value()));
	}

	/**
	 * TODO is this dangerous? All derivations of NewAbstractBounded will have
	 * the same hash code- given they store the same Float - as this Float has.
	 *
	 * For the distinction of the objects one may use the equals(Object) method
	 * though.
	 *
	 * We should look if this has bad consequences with collection usage and
	 * stuff like that, it would point out at least why Float and friends are
	 * final classes, which we by-pass in a manner here.
	 *
	 * XXX (temporal solution): hashCode() not implemented yet, derives the
	 * regular Object.hashCode so far. Let's see if the need for the hashCode
	 * function emerges from the above counted uses.
	 */
	// @Override
	// public int hashCode() {
	// return Float.valueOf(value()).hashCode();
	// }
}
