/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.types;

import java.lang.annotation.Annotation;

@BoundedInteger
public abstract class AbstractBoundedInteger extends Number implements
/** BoundedInteger, */
Comparable<AbstractBoundedInteger> {

	/** {@inheritDoc} */
	public double doubleValue() {
		return Integer.valueOf(value()).doubleValue();
	}

	/** {@inheritDoc} */
	public float floatValue() {
		return Integer.valueOf(value()).floatValue();
	}

	/** {@inheritDoc} */
	public int intValue() {
		return Integer.valueOf(value()).intValue();
	}

	/** {@inheritDoc} */
	public long longValue() {
		return Integer.valueOf(value()).longValue();
	}

	/** {@inheritDoc} */
	public abstract int MAX();

	/** {@inheritDoc} */
	public abstract int MIN();

	/** {@inheritDoc} */
	public abstract int NODATA();

	/** {@inheritDoc} */
	public abstract int value();

	/** {@inheritDoc} */
	public Class<? extends Annotation> annotationType() {
		return BoundedInteger.class.getAnnotation(BoundedInteger.class)
				.annotationType();
	}

	/**
	 * XXX if we wanted to use the signature public int compareTo(BoundedInteger
	 * o) here, we had to add implements BoundedInteger to the sub-class'
	 * signature - replacing too the Comparable<NewAbstractBoundedInteger> part
	 * with Comparable<BoundedInteger>, as well as to change the method
	 * signature in that sub-class.
	 *
	 * @param o
	 * @return int value
	 */
	/** {@inheritDoc} */
	public int compareTo(AbstractBoundedInteger o) {
		return Integer.valueOf(value()).compareTo(Integer.valueOf(o.value()));
	}

	/**
	 * TODO is this dangerous? All derivations of NewAbstractBounded will have
	 * the same hash code- given they store the same Integer - as this Integer
	 * has.
	 *
	 * For the distinction of the objects one may use the equals(Object) method
	 * though.
	 *
	 * We should look if this has bad consequences with collection usage and
	 * stuff like that, it would point out at least why Integer and friends are
	 * final classes, which we by-pass in a manner here.
	 *
	 * XXX (temporal solution): hashCode() not implemented yet, derives the
	 * regular Object.hashCode so far. Let's see if the need for the hashCode
	 * function emerges from the above counted uses.
	 */
	// @Override
	// public int hashCode() {
	// return Integer.valueOf(value()).hashCode();
	// }
}
