/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.types.impl;

import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

public abstract class AbstractSetAdapter<S> implements Set<S> {

	private Set<S> set;

	private Set<S> getSet() {
		return this.set;
	}

	public AbstractSetAdapter(Set<S> set) {
		this.set = set;
	}

	/** {@inheritDoc} */
	public void clear() {
		this.set.clear();
	}

	/** {@inheritDoc} */
	public boolean isEmpty() {
		return getSet().isEmpty();
	}

	/** {@inheritDoc} */
	public int size() {
		return getSet().size();
	}

	/** {@inheritDoc} */
	public boolean contains(Object o) {
		return getSet().contains(o);
	}

	/** {@inheritDoc} */
	public boolean containsAll(Collection<?> c) {
		return getSet().containsAll(c);
	}

	/** {@inheritDoc} */
	public boolean removeAll(Collection<?> c) {
		return this.set.removeAll(c);
	}

	/** {@inheritDoc} */
	public boolean retainAll(Collection<?> c) {
		return this.set.retainAll(c);
	}

	/** {@inheritDoc} */
	public Object[] toArray() {
		return getSet().toArray();
	}

	/** {@inheritDoc} */
	public boolean remove(Object o) {
		return this.set.remove(o);
	}

	/** {@inheritDoc} */
	public Iterator<S> iterator() {
		return getSet().iterator();
	}

	/** {@inheritDoc} */
	public <T> T[] toArray(T[] a) {
		return getSet().toArray(a);
	}

	/** {@inheritDoc} */
	public boolean add(S e) {
		return this.set.add(e);
	}

	/** {@inheritDoc} */
	public boolean addAll(Collection<? extends S> c) {
		return this.set.addAll(c);
	}
}
