/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.types.impl;

import static org.apache.log4j.Logger.getLogger;

import org.apache.log4j.Logger;

import abmland.core.sandbox.BoundsUtil;
import abmland.types.BoundedInteger;
import fj.P1;
import fj.data.Either;

@BoundedInteger(MIN = 0, MAX = (int) 1e1)
public class BoundedIntegerImpl extends Number implements
		Comparable<BoundedIntegerImpl> {

	private static Logger log = getLogger(BoundedIntegerImpl.class);

	private int value;

	private static Class<BoundedInteger> inherited_ = BoundedInteger.class;
	private static Class<BoundedIntegerImpl> this_ = BoundedIntegerImpl.class;

	private static int MIN = this_.isAnnotationPresent(inherited_) ? this_
			.getAnnotation(inherited_).MIN() : null;

	private static int MAX = this_.isAnnotationPresent(inherited_) ? this_
			.getAnnotation(inherited_).MAX() : null;

	public static final Class<Integer> TYPE = Integer.TYPE;

	private BoundedIntegerImpl() {
//		log = getLogger(getClass());
		;
	}

	public BoundedIntegerImpl(int value, final int MIN, final int MAX) {
		BoundedIntegerImpl.MIN = MIN;
		BoundedIntegerImpl.MAX = MAX;
		this.value = guardAssignment(value, MIN, MAX);
	}

	// public BoundedIntegerImpl(int value) {
	// this.value = guardAssignment(value);
	// }

	private static int guardAssignment(int value, final int MIN, final int MAX) {
		int EDOM = Integer.MIN_VALUE;

		P1<Either<Exception, Integer>> lazyE = BoundedIntegerImpl.eAssign(
				value, MIN, MAX);
		int result = (lazyE._1().isLeft() ? EDOM : lazyE._1().right().value());
		if (lazyE._1().isLeft()) {
			log.error("assigned value " + value + " is out of bounds. Assigning " + EDOM, lazyE._1().left().value());
		}
		return result;
	}

	private static P1<Either<Exception, Integer>> eAssign(final Integer value,
			final int MIN, final int MAX) {
		return new P1<Either<Exception, Integer>>() {
			/** {@inheritDoc} */
			public Either<Exception, Integer> _1() {
				try {
					if (BoundsUtil.isOuterBounds(value, MIN, MAX)) {
						throw new Exception(value + " outer bounds (" + MIN + ", " + MAX + ")");
					}
					return Either.right(value);
				} catch (Exception e) {
					return Either.left(e);
				}
			}
		};
	}

	public static void main(String[] args) {
		log.info(BoundedIntegerImpl.valueOf(MIN - 1));
		log.info(BoundedIntegerImpl.valueOf(MIN + 1));
		log.info(BoundedIntegerImpl.valueOf(MAX + 1));
		log.info(BoundedIntegerImpl.valueOf(MAX - 1));
	}

	/**
	 * Returns an Integer object for value of `i`, after checking boundary
	 * constraints
	 *
	 * @param i
	 * @return an Integer object for value of `i`
	 */
	public static Integer valueOf(int i) {
		return new BoundedIntegerImpl(i, MIN, MAX).intValue();
	}

	/** {@inheritDoc} */
	public String toString() {
		return String.valueOf(value);
	}

	/** {@inheritDoc} */
	public double doubleValue() {
		return (double) value;
	}

	/** {@inheritDoc} */
	public float floatValue() {
		return (float) value;
	}

	/** {@inheritDoc} */
	public int intValue() {
		return value;
	}

	/** {@inheritDoc} */
	public long longValue() {
		return (long) value;
	}

	/** {@inheritDoc} */
	public int compareTo(BoundedIntegerImpl o) {
		int thisVal = this.value;
		int anotherVal = o.value;
		return (thisVal < anotherVal ? -1 : (thisVal == anotherVal ? 0 : 1));
	}

	/**
	 * XXX TODO If we leave this out, each `BoundedIntegerImpl`- and
	 * derivative-classified object should have a normal Object `hashCode()`
	 * value as a result
	 */
	/** {@inheritDoc} */
	public int hashCode() {
		return value;
	}
}
