/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.types.impl;

import static abmland.common.StringUtils.charr2String;
import static abmland.data.CLCover.randomValue;
import repast.simphony.valueLayer.GridValueLayer;
import abmland.data.CLCover;
import abmland.mainctrl.config.ConfManager;
import abmland.mainctrl.config.MiscConf;
import abmland.types.ICLCoverAllCells;

/**
 * Basically un-checked
 *
 * @author dkahlenb
 *
 */
public class CLCoverAllCells extends AbstractAllCells<CLCover> implements
		ICLCoverAllCells {

	private static final String NAME = "CLCoverAllCellsGridVL";

	private static int[] dimensions = ConfManager.MISCCONF
			.getConfig(MiscConf.class).areaParameters.getDims();

	/**
	 * @tag usage.protocol : Assumes that the given layer has the correct
	 *      dimensions.
	 */
	public CLCoverAllCells(GridValueLayer layer) {
		super(layer);
	}

	/**
	 *
	 * @param value
	 * @param name of underlying grid value layer - optional
	 */
	public CLCoverAllCells(CLCover value, String... name) {
		// boundary checking inclusive
		super(new GridValueLayer(charr2String(NAME, name), value.value(), true,
				dimensions));
	}

	/**
	 *
	 * @param value
	 * @param name of underlying grid value layer - optional
	 */
	public CLCoverAllCells(Double value, String... name) {
		// boundary checking inclusive
		super(new GridValueLayer(charr2String(NAME, name), CLCover.valueOf(
				value).value(), true, dimensions));
	}

	public CLCoverAllCells(String... name) {
		super(new GridValueLayer(charr2String(NAME, name), randomValue()
				.value(), true, dimensions));
	}

	/** {@inheritDoc} */
	public final void set(CLCover value, int... coordinate) {
		try {
			super.set(value.value(), coordinate);
		} catch (Exception e) {
			System.err.println(value != null);
			System.err.println(String.format("%s, %s", coordinate[0],coordinate[1]));
			
		}
	}
}
