/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.types.impl;

import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import repast.simphony.space.grid.GridPoint;
import abmland.data.CLCover;
import abmland.types.ICLCoversAllCells;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;
import com.google.common.collect.Multiset;

/**
 * Is a HashMultimap&lt;GridPoint, CLCover&gt;
 *
 * @author dkahlenb
 *
 */
public class CLCoversAllCells implements ICLCoversAllCells {

		private HashMultimap<GridPoint, CLCover> hashMultimap;

		public CLCoversAllCells(Multimap<GridPoint, CLCover> map) {
			hashMultimap = HashMultimap.create(map);
		}

		public CLCoversAllCells() {
			hashMultimap = HashMultimap.create();
		}

		public Map<GridPoint, Collection<CLCover>> asMap() {
			return hashMultimap.asMap();
		}

		public void clear() {
			hashMultimap.clear();
		}

		public boolean containsEntry(Object key, Object value) {
			return hashMultimap.containsEntry(key, value);
		}

		public boolean containsKey(Object key) {
			return hashMultimap.containsKey(key);
		}

		public boolean containsValue(Object value) {
			return hashMultimap.containsValue(value);
		}

		public Collection<Entry<GridPoint, CLCover>> entries() {
			return hashMultimap.entries();
		}

		public Collection<CLCover> get(GridPoint key) {
			return hashMultimap.get(key);
		}

		public boolean isEmpty() {
			return hashMultimap.isEmpty();
		}

		public Set<GridPoint> keySet() {
			return hashMultimap.keySet();
		}

		public Multiset<GridPoint> keys() {
			return hashMultimap.keys();
		}

		public boolean put(GridPoint key, CLCover value) {
			return hashMultimap.put(key, value);
		}

		public boolean putAll(
				Multimap<? extends GridPoint, ? extends CLCover> multimap) {
			return hashMultimap.putAll(multimap);
		}

		public boolean putAll(GridPoint key,
				Iterable<? extends CLCover> values) {
			return hashMultimap.putAll(key, values);
		}

		public boolean remove(Object key, Object value) {
			return hashMultimap.remove(key, value);
		}

		public Collection<CLCover> removeAll(Object key) {
			return hashMultimap.removeAll(key);
		}

		public Collection<CLCover> replaceValues(GridPoint key,
				Iterable<? extends CLCover> values) {
			return hashMultimap.replaceValues(key, values);
		}

		public int size() {
			return hashMultimap.size();
		}

		public Collection<CLCover> values() {
			return hashMultimap.values();
		}
}
