/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.types.impl;

import static abmland.common.LayerEvent.OBJECT_ADDED;
import static abmland.common.LayerEvent.OBJECT_INVALID;
import static abmland.common.StringUtils.charr2String;
import static org.apache.log4j.Logger.getLogger;

import java.util.Collection;

import org.apache.log4j.Logger;

import repast.simphony.valueLayer.GridValueLayer;
import abmland.common.CellDumbMock;
import abmland.common.LayerEvent;
import abmland.common.LayerListener;
import abmland.mainctrl.config.ConfManager;
import abmland.mainctrl.config.MiscConf;
import abmland.types.ILandPriceIndexAllCells;
import abmland.types.IPriceIndex;

public class LandPriceIndexAllCells extends AbstractAllCells<IPriceIndex>
		implements ILandPriceIndexAllCells {

	/**
	 * @tag usage.protocol : Assumes that everything
	 *      (src/main/resources/config/params/.../MiscConf.xml) is initialized
	 *      correctly to retrieve the configuration driven default dimensions.
	 */
	private static int[] dimensions = ConfManager.MISCCONF
			.getConfig(MiscConf.class).areaParameters.getDims();

	private static final String NAME = "LandPriceIndexAllCellsGridVL";

	private static final Logger log = getLogger(LandPriceIndexAllCells.class);

	/**
	 * @tag usage.protocol : Assumes that the given layer has the correct
	 *      dimensions.
	 */
	public LandPriceIndexAllCells(GridValueLayer layer) {
		super(layer);
	}

	/**
	 * @param name
	 *            of underlying grid value layer - optional
	 * @tag usage.protocol : Assumes that everything
	 *      (src/main/resources/config/params/.../MiscConf.xml) is initialized
	 *      correctly to retrieve the configuration driven default dimensions.
	 */
	public LandPriceIndexAllCells(PriceIndex bounded, String... name) {
		super(new GridValueLayer(charr2String(NAME, name), bounded.value(),
				true, dimensions));
	}

	/**
	 * @param name
	 *            of underlying grid value layer - optional
	 * @tag usage.protocol : Assumes that everything
	 *      (src/main/resources/config/params/.../MiscConf.xml) is initialized
	 *      correctly to retrieve the configuration driven default dimensions.
	 */
	public LandPriceIndexAllCells(Double value, String... name) {
		// boundary checking inclusive
		super(new GridValueLayer(charr2String(NAME, name), PriceIndex
				.valueOf(value), true, dimensions));
	}

	/**
	 * @param name
	 *            of underlying grid value layer - optional
	 * @tag usage.protocol : Assumes that everything
	 *      (src/main/resources/config/params/.../MiscConf.xml) is initialized
	 *      correctly to retrieve the configuration driven default dimensions.
	 */
	public LandPriceIndexAllCells(String... name) {
		super(new GridValueLayer(charr2String(NAME, name), PriceIndex.MIN,
				true, dimensions));
	}

	/** {@inheritDoc} */
	public void set(IPriceIndex value, int... coordinate) {
		super.set(value.value(), coordinate);
	}

	public void set(double value, int... coordinate) {
		try {
//			log.trace(value);
			double floatvalue = PriceIndex.valueOf(value, false);
			/**
			 * Only store if no [exception thrown until now] boundaries broken
			 */
			super.set(floatvalue, coordinate);
			layerEventOccurred(new LayerEvent<IPriceIndex>(this, value,
					OBJECT_ADDED));

		} catch (Throwable e) {
			layerEventOccurred(new LayerEvent<IPriceIndex>(this, value,
					OBJECT_INVALID));
			e.printStackTrace();
			System.exit(12);
		}
	}

	/** {@inheritDoc} */
	public void layerEventOccurred(LayerEvent<IPriceIndex> evt) {
		Collection<LayerListener<IPriceIndex>> listeners = getLayerListeners();
		for (LayerListener<IPriceIndex> layerListener : listeners) {
			layerListener.layerEventOccurred(evt);
		}
	}
}
