/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.types.impl;

import static abmland.types.impl.TypeUtil.guardAssignment;
import abmland.types.BoundedDouble;
import abmland.types.AbstractBoundedDouble;
import abmland.types.ISupply;

@BoundedDouble(MIN = 0.0f, MAX = 1e6f, NODATA = Double.NaN)
public final class Supply extends AbstractBoundedDouble implements ISupply {

	private double value = Supply.NODATA;

	// if PERIPHERIC is true, this value relates to a non-CS area grid point and
	// Double.NaN is
	// allowed as internal representation, if not, an exception should be
	// thrown, marking
	// the Double.NaN to be result of failure (division by zero etc.).
	private boolean PERIPHERIC = false;

	private static Class<BoundedDouble> inherited_ = BoundedDouble.class;
	private static Class<Supply> this_ = Supply.class;

	public static final Double MIN = this_.isAnnotationPresent(inherited_) ? this_
			.getAnnotation(inherited_).MIN()
			: null;

	public static final Double MAX = this_.isAnnotationPresent(inherited_) ? this_
			.getAnnotation(inherited_).MAX()
			: null;

	public final static Double NODATA = this_.isAnnotationPresent(inherited_) ? this_
			.getAnnotation(inherited_).NODATA()
			: null;

	public Supply(Number value, boolean... fromPeriphery)
			throws RuntimeException {
		// in this first case, NaN is stored
		if (fromPeriphery.length > 0 && fromPeriphery[0]) {
			this.PERIPHERIC = true;
			try {
				this.value = guardAssignment(value.doubleValue(), MIN, MAX,
						NODATA);
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		} else if ((fromPeriphery.length > 0 && !fromPeriphery[0])
				|| fromPeriphery.length <= 0) {
			this.PERIPHERIC = false;
			try {
				this.value = guardAssignment(value.doubleValue(), MIN, MAX, MAX);
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}
	}

	public Supply(boolean... fromPeriphery) {
		this.value = NODATA;
		// in this first case, NaN is stored
		if (fromPeriphery.length > 0 && fromPeriphery[0]) {
			this.PERIPHERIC = true;
			this.value = NODATA;
		} else if ((fromPeriphery.length > 0 && !fromPeriphery[0])
				|| 0 >= fromPeriphery.length) {
			this.PERIPHERIC = false;
			this.value = MIN;
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.types.impl.ISupply#MAX()
	 */
	public double MAX() {
		return Supply.MAX;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see abmland.types.impl.ISupply#MIN()
	 */
	public double MIN() {
		return Supply.MIN;
	}

	@Override
	public double NODATA() {
		return Supply.NODATA;
	}

	/** {@inheritDoc} */
	public double value() {
		return value;
	}

	/**
	 * Need bound checking here, means an instance created, so this cannot be in
	 * the abstract super class.
	 *
	 * @param f
	 * @return a double value stored by this instance off NewSupply
	 */
	public final static Double valueOf(double f, boolean... fromPeriphery)
			throws RuntimeException {
		return new Supply(f, fromPeriphery).value();
	}

	// TODO Return a BoundedDouble or NewSupply here?
	public static Supply getInstance(Number n, boolean... fromPeriphery)
			throws RuntimeException {
		return new Supply(n.doubleValue(), fromPeriphery);
	}

	/** {@inheritDoc} */
	public int compareTo(AbstractBoundedDouble another) {
		return super.compareTo(another);
	}

	/** {@inheritDoc} */
	public boolean equals(Object obj) {
		return (obj instanceof Supply) && value() == ((Supply) obj).value();
	}
}
