/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.types.impl;

import static abmland.common.LayerEvent.OBJECT_ADDED;
import static abmland.common.LayerEvent.OBJECT_INVALID;
import static abmland.common.StringUtils.charr2String;

import java.util.Collection;

import repast.simphony.valueLayer.GridValueLayer;
import abmland.common.LayerEvent;
import abmland.common.LayerListener;
import abmland.mainctrl.config.ConfManager;
import abmland.mainctrl.config.MiscConf;
import abmland.types.ISupply;
import abmland.types.ISupplyAllCells;

public class SupplyAllCells extends AbstractAllCells<ISupply> implements
		ISupplyAllCells {

	/**
	 * @tag usage.protocol : Assumes that everything
	 *      (src/main/resources/config/params/.../MiscConf.xml) is initialized
	 *      correctly to retrieve the configuration driven default dimensions.
	 */
	private static int[] dimensions = ConfManager.MISCCONF
			.getConfig(MiscConf.class).areaParameters.getDims();

	private static final String NAME = "SupplyAllCellsGridVL";

	/**
	 * @tag usage.protocol : Assumes that the given layer has the correct
	 *      dimensions.
	 */
	public SupplyAllCells(GridValueLayer layer) {
		super(layer);
	}

	/**
	 * @param name
	 *            of underlying grid value layer - optional
	 * @tag usage.protocol : Assumes that everything
	 *      (src/main/resources/config/params/.../MiscConf.xml) is initialized
	 *      correctly to retrieve the configuration driven default dimensions.
	 */
	public SupplyAllCells(Supply bounded, String... name) {
		super(new GridValueLayer(charr2String(NAME, name), bounded.value(),
				true, dimensions));
	}

	/**
	 * @param name
	 *            of underlying grid value layer - optional
	 * @tag usage.protocol : Assumes that everything
	 *      (src/main/resources/config/params/.../MiscConf.xml) is initialized
	 *      correctly to retrieve the configuration driven default dimensions.
	 */
	public SupplyAllCells(Double value, String... name) {
		// boundary checking inclusive
		super(new GridValueLayer(charr2String(NAME, name), Supply
				.valueOf(value), true, dimensions));
	}

	/**
	 * @param name
	 *            of underlying grid value layer - optional
	 * @tag usage.protocol : Assumes that everything
	 *      (src/main/resources/config/params/.../MiscConf.xml) is initialized
	 *      correctly to retrieve the configuration driven default dimensions.
	 */
	public SupplyAllCells(String... name) {
		super(new GridValueLayer(charr2String(NAME, name), Supply.MIN, true,
				dimensions));
	}

	/** {@inheritDoc} */
	public void set(ISupply value, int... coordinate) {
		super.set(value.value(), coordinate);
	}

	/** {@inheritDoc} */
	public void set(double value, int... coordinate) {
		try {
			double floatvalue = Supply.valueOf(value);
			/**
			 * Only store if no [exception thrown until now] boundaries broken
			 */
			super.set(floatvalue, coordinate);
			layerEventOccurred(new LayerEvent<ISupply>(this, value,
					OBJECT_ADDED));

		} catch (Throwable e) {
			layerEventOccurred(new LayerEvent<ISupply>(this, value,
					OBJECT_INVALID));
			e.printStackTrace();
		}
	}

	public void layerEventOccurred(LayerEvent<ISupply> evt) {
		Collection<LayerListener<ISupply>> listeners = getLayerListeners();
		for (LayerListener<ISupply> layerListener : listeners) {
			layerListener.layerEventOccurred(evt);
		}
	}
}
