/**
 * Copyright (C) 2010 [[XYZ]], Helmholtz Centre for Environmental Research GmbH - UFZ <abmland@ufz.de>
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package abmland.types.impl;

//import static org.apache.log4j.Logger.getLogger;

//import org.apache.log4j.Logger;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

import fj.P1;
import fj.data.Either;

public class TypeUtil {

//	private static Logger log = getLogger(TypeUtil.class);

	@Rule
	public ExpectedException thrown = ExpectedException.none();

	private static String msg = "%s outer bounds (%s, %s) and not NODATA (%s)";

	@Test
	public void throwsExceptionBecauseOutOfDefinitionRange() throws Exception {
		Float value = 1.1f, MIN = 0f, MAX = 1.0f, NODATA = Float.NaN;
		thrown.expect(Exception.class);
		thrown.expectMessage(String.format(msg, value, MIN, MAX, NODATA));
		guardAssignment(value, MIN, MAX, NODATA);
	}

	@Test
	public void successfulAssignment() throws Exception {
		Float value = 1.1f, MIN = 0f, MAX = 1.0f, NODATA = 1.1f;
		guardAssignment(value, MIN, MAX, NODATA);
	}

	public final static <T extends Comparable<T>> T guardAssignment(T value,
			final T MIN, final T MAX, final T NODATA) throws Exception {
		// T EDOM = null;

		P1<Either<Exception, T>> lazyE = eAssign(value, MIN, MAX, NODATA);

		T result = (lazyE._1().isLeft() ? /** EDOM */
		NODATA : lazyE._1().right().value());

		if (lazyE._1().isLeft()) {

			// String logmsg =
			// String.format("assigned value %s is out of bounds" //
			// /**
			// *
			// +". Assigning %s. This a-priori assignment is not necessarily good, though null may be obviously as well as faithful-to-truth enough. TODO remove it"
			// */
			// + ".", value /** , EDOM */
			// );
			//
			// log.fatal(logmsg, lazyE._1().left().value());

			// re-throw the exception...
			throw lazyE._1().left().value();
		}
		return result;
	}

	private static <T extends Comparable<T>> P1<Either<Exception, T>> eAssign(
			final T value, final T MIN, final T MAX, final T NODATA) {
		return new P1<Either<Exception, T>>() {
			/** {@inheritDoc} */
			public Either<Exception, T> _1() {
				try {
					if (isOuterBounds(value, MIN, MAX, NODATA)) {
						// A more specific will be thrown later in the monad,
						// this is for triggering the Either.left with Exception
						// e retournament.
						throw new Exception(String.format(msg, value, MIN, MAX,
								NODATA));
					}
					return Either.right(value);
				} catch (Exception e) {
					return Either.left(e);
				}
			}
		};
	}

	/**
	 * Given when the value is outside MIN or MAX and is not NODATA
	 *
	 * @param <T>
	 * @param value
	 * @param lowerBound
	 * @param upperBound
	 * @return
	 */
	private static <T extends Comparable<T>> boolean isOuterBounds(T value,
			T lowerBound, T upperBound, T noData) {
		// log.fatal(value.compareTo(noData) != 0);
		// if it is not NODATA and it is outer MIN,MAX
		if (value.compareTo(noData) != 0 && // short-circuiting here!
				// Parentheses very important:
				(value.compareTo(lowerBound) < 0 || value.compareTo(upperBound) > 0)) {
			return true;
		}
		return false;
	}
}
